import os

import unittest

test_dir = os.path.dirname(os.path.realpath(__file__))
scripts_dir = test_dir.replace('/tests', '/scripts')

import sys

sys.path.insert(0, scripts_dir)

from settings import *


# if not os.path.exists('output'):
#     os.makedirs('output')
#     #enter inside folder
# os.chdir('output')

# rm -rf cadd_vest hgmd hi_index merge output pyannotator sanity_check snpeff snpsift validator vep

class TestAnnotator(unittest.TestCase):
    def setUp(self):
        self.tests_dir = os.path.dirname(os.path.realpath(__file__))
        self.scripts_dir = self.tests_dir.replace('tests', 'scripts')

        self.sample_name = 'sample.100.vcf'
        self.sample = self.scripts_dir.replace('scripts', 'samples/sample.100.vcf')  # sample_proton.vcf

    def test_001_validator(self):
        command = 'python %s/validator.py -i %s' % (self.scripts_dir, self.sample)
        print('command running', command)
        os.system(command)
        # now check if it worked!

    def test_002_sanity_check(self):
        command = 'python %s/sanity_check.py -i %s' % (self.scripts_dir, self.sample)
        os.system(command)
        # now check if it worked!
        fname = 'sanity_check/checked.vcf'
        self.assertTrue(os.path.isfile(fname))

    def test_003_snpeff(self):
        command = 'python %s/snpeff.py -i %s' % (self.scripts_dir, self.sample)
        os.system(command)
        # now check if it worked!
        fname = 'snpeff/snpeff.output.vcf'
        self.assertTrue(os.path.isfile(fname))

    def test_004_vep(self):
        command = 'python %s/vep.py -i %s' % (self.scripts_dir, self.sample)
        os.system(command)
        # now check if it worked!
        fname = 'vep/vep.output.sorted.vcf'
        self.assertTrue(os.path.isfile(fname))

    def test_005_hi_index(self):
        command = 'python %s/hi_index.py -i %s' % (self.scripts_dir, self.sample)
        os.system(command)
        # now check if it worked!
        fname = 'hi_index/hi_index.vcf'
        self.assertTrue(os.path.isfile(fname))

    def test_006_hgmd(self):
        command = 'python %s/hgmd.py -i %s' % (self.scripts_dir, self.sample)
        os.system(command)
        # now check if it worked!
        fname = 'hgmd/hgmd.vcf'
        self.assertTrue(os.path.isfile(fname))

    def test_007_snpsift(self):
        command = 'python %s/snpsift.py -i %s' % (self.scripts_dir, self.sample)
        os.system(command)
        # now check if it worked!
        fname = 'snpsift/snpsift.final.vcf'
        self.assertTrue(os.path.isfile(fname))

    def test_008_vcf_annotator(self):
        # command = 'python %s/vcf_annotator.py -i %s' % (self.scripts_dir, self.sample)
        command = '''python %s/vcf_annotator_parallel.py \
        -i %s \
        -n %s \
        -r 1000genomes dbsnp clinvar esp6500 \
        -a %s %s %s %s''' % (self.scripts_dir, self.sample, pynnotator_cores, genomes1k, dbsnp, clinvar, esp)

        os.system(command)
        # now check if it worked!
        fname = 'pynnotator/pynnotator.vcf'
        self.assertTrue(os.path.isfile(fname))

    def test_009_cadd_vest(self):
        command = '''python %s/cadd_vest_parallel.py \
        -i %s \
        -n %s''' % (self.scripts_dir, self.sample, cadd_vest_cores)

        os.system(command)
        # now check if it worked!
        fname = 'cadd_vest/cadd_vest.vcf'
        self.assertTrue(os.path.isfile(fname))

    def test_010_merge(self):
        #     command = 'python %s/merge.py \
        #     -i ' % (self.scripts_dir, self.sample)
        command = 'python %s/merge.py -i sanity_check/checked.vcf' % (scripts_dir)
        os.system(command)
        #     #now check if it worked!
        #     fname = 'cadd_vest/cadd_vest.vcf'
        #     self.assertTrue(os.path.isfile(fname))


        # def test_shuffle(self):
        #     # make sure the shuffled sequence does not lose any elements
        #     random.shuffle(self.seq)
        #     self.seq.sort()
        #     self.assertEqual(self.seq, range(10))

        #     # should raise an exception for an immutable sequence
        #     self.assertRaises(TypeError, random.shuffle, (1,2,3))

        # def test_choice(self):
        #     element = random.choice(self.seq)
        #     self.assertTrue(element in self.seq)

        # def test_sample(self):
        #     with self.assertRaises(ValueError):
        #         random.sample(self.seq, 20)
        #     for element in random.sample(self.seq, 5):
        #         self.assertTrue(element in self.seq)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# parallel https://code.google.com/p/pysam/issues/detail?id=105
# import pp

import argparse
import multiprocessing as mp
import os
import pysam
from datetime import datetime
from pynnotator import settings
from subprocess import run

class Dbnsfp(object):
    def __init__(self, vcf_file=None, cores=None):
    
        self.vcf_file = vcf_file
        self.dbnsfp_header = open('%s/dbnsfp/header.vcf' % (settings.data_dir)).readlines()

        # print('self.resources', self.resources)
        self.cores = int(cores)

        self.filename = os.path.splitext(os.path.basename(str(vcf_file)))[0]
        # create folder validator if it doesn't exists
        if not os.path.exists('dbnsfp'):
            os.makedirs('dbnsfp')

    def run(self):

        tstart = datetime.now()

        print(tstart, 'Starting DBNSFP annotator: ', self.vcf_file)

        # std = self.annotator()

        self.splitvcf(self.vcf_file)

        pool = mp.Pool()
        pool.map(self.annotate, range(1, self.cores + 1))
        # pool.close()
        # pool.join()

        prefix = 'dbnsfp'
        # # Define your jobs
        # jobs = []
        final_parts = []
        for n in range(0, self.cores):
            index = n + 1
            final_file = 'dbnsfp/dbnsfp.%s.vcf' % (index)
            final_parts.append(final_file)

        command = 'cat %s/header.vcf ' % (prefix) + " ".join(final_parts) + '> %s/dbnsfp.vcf' % (prefix)
        run(command, shell=True)

        command = 'rm %s/header.vcf %s/body.vcf %s/dbnsfp.*.vcf %s/part.*' % (prefix, prefix, prefix, prefix)
        # run(command, shell=True)

        tend = datetime.now()
        annotation_time = tend - tstart
        print(tend, 'Finished DBNSFP, it took: ', annotation_time)

    def partition(self, lst, n):
        division = len(lst) / float(n)
        return [lst[int(round(division * i)): int(round(division * (i + 1)))] for i in range(n)]

    def splitvcf(self, vcffile):
        # print('split file', vcffile)
        # print 'numero de cores', cores
        prefix = 'dbnsfp'
        vcf_reader = open('%s' % (vcffile))
        header_writer = open('%s/header.vcf' % (prefix), 'w')
        body_writer = open('%s/body.vcf' % (prefix), 'w')

        count_lines = 0
        for line in vcf_reader:
            if line.startswith('#'):
                if line.startswith('#CHROM'):
                    header_writer.writelines(self.dbnsfp_header)
                header_writer.writelines(line)
            else:
                body_writer.writelines(line)
        header_writer.close()
        body_writer.close()

        vcf_reader = open('%s/body.vcf' % (prefix))

        groups = self.partition(list(vcf_reader.readlines()), self.cores)
        for c, group in enumerate(groups):
            # print 'group', len(group)
            # print 'c', c
            part = c + 1
            part_writer = open('%s/part.%s.vcf' % (prefix, part), 'w')
            for line in group:
                part_writer.writelines(line)
            part_writer.close()

    # convert and annotate the vcf file to snpeff
    def annotate(self, out_prefix):
        # print 'Hello'
        # print self.dbnsfp_reader
        # header is at:

        # 24    SIFT_score: SIFT score (SIFTori).
        # 105 HUVEC_confidence_value: 0 - highly significant scores (approx. p<.003); 1 - significant scores

        # 188   clinvar_rs: rs number from the clinvar data set
        # 191 clinvar_golden_stars: ClinVar Review Status summary.

        # print 'input',vcffile, out_prefix, dbnsfp 
        dbnsfp_reader = pysam.Tabixfile(settings.dbnsfp, 'r', encoding='utf-8')

        # print('header')
        for item in dbnsfp_reader.header:
            header = item.strip().split('\t')

        # header = dbnsfp_reader.header.next().strip().split('\t')

        vcffile = 'dbnsfp/part.%s.vcf' % (out_prefix)

        vcf_reader = open('%s' % (vcffile))
        vcf_writer = open('dbnsfp/dbnsfp.%s.vcf' % (out_prefix), 'w', encoding="utf-8")

        for line in vcf_reader:
            if line.startswith('#'):
                if line.startswith('#CHROM'):
                    vcf_writer.writelines(dbnsfp_header)
                vcf_writer.writelines(line)
            else:
                variant = line.split('\t')
                variant[0] = variant[0].replace('chr', '')
                index = '%s-%s' % (variant[0], variant[1])
                # print index
                try:
                    records = dbnsfp_reader.fetch(variant[0], int(variant[1]) - 1, int(variant[1]),multiple_iterators=True)
                except:
                    records = []

                for record in records:
                    ann = record.strip().split('\t')

                    ispresent = False
                    if variant[3] == ann[2]:
                        alts = variant[4].split(',')
                        alts_ann = ann[3].split(',')
                        # compare ALT
                        for alt in alts:
                            if alt in alts_ann:
                                ispresent = True

                    if ispresent:
                        new_ann = []
                        for k, item in enumerate(header):
                            idx = k
                            if ann[idx] != '.':
                                new_ann.append('dbNSFP_%s=%s' % (item, ann[idx].replace(';', '|')))
                        variant[7] = '%s;%s' % (variant[7], ";".join(new_ann))
                vcf_writer.writelines("\t".join(variant))
        vcf_writer.close()


if __name__ == '__main__':
    
    parser = argparse.ArgumentParser(description='Annotate a VCF File with DbNSFP.')
    parser.add_argument('-i', dest='vcf_file', required=True, metavar='example.vcf', help='a VCF file to be annotated')
    parser.add_argument('-n', dest='cores', required=True, metavar='4', help='number of cores to use')
    args = parser.parse_args()
    dbnfsp = Dbnsfp(args.vcf_file, args.cores)
    dbnfsp.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import argparse
import os
import subprocess
from subprocess import call, run, check_output
from datetime import datetime
import logging

from pynnotator import settings


toolname = 'validator'

env = os.environ.copy()
env['PERL5LIB'] = settings.vcftools_dir_perl

class Validator(object):
    def __init__(self, vcf_file=None):

        self.vcf_file = vcf_file
        self.filename = os.path.splitext(os.path.basename(str(vcf_file)))[0]
        
        logging.basicConfig(filename='validation.log',level=logging.DEBUG)

        # create folder validator if it doesn't exists
        if not os.path.exists('validator'):
            os.makedirs('validator')

    def log(self, msg):
        
        logging.info(msg)
        print(msg)


    def run(self):

        tstart = datetime.now()
        msg = "{} Starting validator: {}".format(tstart, self.vcf_file)
        
        self.log(msg)

        std = self.validate()

        tend = datetime.now()
        annotation_time = tend - tstart
        
        msg = "{} Finished validator, it took: {}".format(tend, annotation_time)
        self.log(msg)

        return std

    # Validate vcf file with Vcftools
    def validate(self):
        # check if file is in .gz format

        # if not self.vcf_file.endswith('.gz'):
        #     command = '%s/bgzip -c %s > validator/%s.vcf.gz' % (settings.htslib_dir, self.vcf_file, self.filename)
        #     call(command, shell=True)
        # else:
        command = 'cp %s validator/' % (self.vcf_file)
        call(command, shell=True)

        # command = '%s/tabix -p vcf validator/%s.vcf.gz' % (settings.htslib_dir, self.filename)
        # call(command, shell=True)
        
        command = '%s/vcf_validator -i validator/%s.vcf 1>validator/validation.log 2>&1' % (
            settings.vcf_validator_dir, self.filename)

        try:
            output = check_output(command, shell=True)
            logging.info('Command Output: %s' % (output))
        except subprocess.CalledProcessError as e:
            
            msg =  "Error {}".format(e.output.decode('utf-8'))
            self.log(msg)

        command = 'tail validator/validation.log'
        call(command, shell=True)        
                
        time_end = datetime.now()
        command = 'rm validator/%s.vcf' % (self.filename)
        run(command, shell=True)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Annotate a VCF File with VAlIDATOR.')
    parser.add_argument('-i', dest='vcf_file', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()
    validator = Validator(args.vcf_file)
    validator.run()



#!/usr/bin/env python3

import argparse
import pynnotator

from subprocess import run, check_call

parser = argparse.ArgumentParser()
parser.add_argument('options', help='install test', nargs='?')
parser.add_argument('-i', dest='vcf_file', required=False, metavar='example.vcf', help='a VCF file to be annotated')

args = parser.parse_args()

def main():

    if args.options == 'update':
        print('Updating Pynnotator to latest version...')
        output = check_call('pip install -U pynnotator', shell=True)
        if output == 0:
            print('Pynnotator was updated!')
    if args.options == 'install':
        obj = pynnotator.Pynnotator()
        obj.install()
    elif args.options == 'install_libs':
        obj = pynnotator.Pynnotator()
        obj.install_libs()
    elif args.options == 'install_requirements':
        obj = pynnotator.Pynnotator()
        obj.install_requirements()
    elif args.options == 'build':
        obj = pynnotator.Pynnotator()
        obj.build()
    elif args.vcf_file:
        obj = pynnotator.Pynnotator()
        obj.annotate(args.vcf_file)
    elif args.options == 'test':
        obj = pynnotator.Pynnotator()
        obj.test()
    elif args.options == 'vep':
        obj = pynnotator.Pynnotator()
        obj.vep()


#!/usr/bin/python
# -*- coding: utf-8 -*-
# test vcf annotation in parallel on python 3

import argparse
import gzip
import multiprocessing as mp
import os
import pysam
from datetime import datetime
from subprocess import run

class VCF_Annotator(object):
    def __init__(self, vcf_file=None, ann_files=None, resources=None, cores=None):

        self.vcf_file = vcf_file

        self.ann_files = ann_files
        # print('self.ann_files', self.ann_files)

        self.resources = resources
        # print('self.resources', self.resources)
        self.cores = int(cores)

        self.filename = os.path.splitext(os.path.basename(str(vcf_file)))[0]
        # create folder validator if it doesn't exists
        if not os.path.exists('pynnotator'):
            os.makedirs('pynnotator')

        ann_dict = {}
        self.annheader = {}

        for key, annfile in enumerate(self.ann_files):
            annfile = gzip.open(annfile, 'rt')
            ann_dict[key] = {}
            self.annheader[key] = []
            for line in annfile:
                if line.startswith('#'):
                    if line.startswith('##INFO'):
                        row = line.split('=')
                        # this treats HGMD-PUBLIC_20152
                        row[2] = row[2].replace('HGMD-', 'HGMD_')

                        row[2] = '%s.%s' % (self.resources[key], row[2])
                        info = "=".join(row)
                        self.annheader[key].append(info)
                else:
                    break

    def run(self):

        tstart = datetime.now()

        print(tstart, 'Starting python vcf annotator: ', self.vcf_file)

        # std = self.annotator()

        self.splitvcf(self.vcf_file, self.annheader)

        pool = mp.Pool()
        pool.map(self.annotate, range(1, self.cores + 1))
        # pool.close()
        # pool.join()

        prefix = 'pynnotator'
        # # Define your jobs
        # jobs = []
        final_parts = []
        for n in range(0, self.cores):
            index = n + 1
            final_file = 'pynnotator/pynnotator.%s.vcf' % (index)
            final_parts.append(final_file)

        command = 'cat %s/header.vcf ' % (prefix) + " ".join(final_parts) + '> %s/pynnotator.vcf' % (prefix)
        run(command, shell=True)

        command = 'rm %s/header.vcf %s/body.vcf %s/pynnotator.*.vcf %s/part.*' % (prefix, prefix, prefix, prefix)
        # run(command, shell=True)


        tend = datetime.now()
        annotation_time = tend - tstart
        print(tend, 'Finished python vcf annotator, it took: ', annotation_time)

    def partition(self, lst, n):
        division = len(lst) / float(n)
        return [lst[int(round(division * i)): int(round(division * (i + 1)))] for i in range(n)]

    def splitvcf(self, vcf_file, annheader):
        # print 'numero de cores', cores
        prefix = 'pynnotator'
        vcf_reader = open('%s' % (vcf_file))
        header_writer = open('%s/header.vcf' % (prefix), 'w')
        body_writer = open('%s/body.vcf' % (prefix), 'w')

        count_lines = 0
        for line in vcf_reader:
            if line.startswith('#'):
                if line.startswith('#CHROM'):
                    for key in annheader:
                        header_writer.writelines(annheader[key])
                header_writer.writelines(line)
            else:
                body_writer.writelines(line)
        header_writer.close()
        body_writer.close()

        # this is a hack to split a vcf into n equal parts!
        command = 'split -d -l$((`wc -l < %s/body.vcf`/%s)) %s/body.vcf pynnotator/part.' % (prefix, self.cores, prefix)
        run(command, shell=True)

        # vcf_reader = open('%s/body.vcf' % (prefix))

        # groups = self.partition(list(vcf_reader.readlines()), self.cores)
        # for c, group in enumerate(groups):
        #     # print 'group', len(group)
        #     # print 'c', c
        #     part = c + 1
        #     part_writer = open('%s/part.%s.vcf' % (prefix, part), 'w')
        #     for line in group:
        #         part_writer.writelines(line)
        #     part_writer.close()

    # convert and annotate the vcf file to snpeff
    def annotate(self, out_prefix):

        # print('out_prefix', out_prefix)

        annfiles = {}

        for n, value in enumerate(self.ann_files):
            annfiles[n] = {}
            annfiles[n]['resource'] = self.resources[n]
            annfiles[n]['file'] = value
            annfiles[n]['reader'] = pysam.Tabixfile(annfiles[n]['file'], encoding="utf-8")

        # represent an integer with two digits
        vcf_file = 'pynnotator/part.%02d' % (out_prefix - 1)

        vcflist = []
        # read first vcf to be annotated
        # print 'vcf_file', vcf_file
        vcf_file = open('%s' % (vcf_file), 'r')

        outvcf_file = open('pynnotator/pynnotator.%s.vcf' % (out_prefix), 'w', encoding="utf-8")

        for line in vcf_file:
            if not line.startswith('#'):

                variant = line.split('\t')
                variant[0] = variant[0].replace('chr', '')
                index = '%s-%s' % (variant[0], variant[1])
                # print 'index', index

                for key, annfile in annfiles.items():
                    # vcf_reader = vcf.Reader(filename=annfile['file'])
                    try:
                        records = annfiles[key]['reader'].fetch(variant[0], int(variant[1]) - 1, int(variant[1]))
                    except:
                        records = []

                    for record in records:

                        row = record.split('\t')

                        # compare REF and ALT Columns to check if they are the same
                        # ex. G C can also be C G in another VCF
                        # ex2. G C,A should match also G A and G C

                        check_flag = False
                        if variant[3] == row[3]:
                            alts = variant[4].split(',')
                            alts_row = row[4].split(',')
                            # compare ALT
                            for alt in alts:
                                if alt in alts_row:
                                    check_flag = True

                        if check_flag:

                            info = row[7].split(';')
                            new_ann = []
                            for k in info:

                                # treat k for snpsift
                                # k = k.replace('=)',')')
                                # k = k.replace('http://www.ncbi.nlm.nih.gov/pubmed?term=','PMID')
                                # remove equal signs (for snpsift)
                                # = for %3D
                                if k.count('=') > 1:
                                    new_string = k.split('=', 1)
                                    # print(new_string)
                                    new_string[1] = new_string[1].replace('=', '')
                                    k = '%s=%s' % (new_string[0], new_string[1])
                                    # print(k)
                                k = k.replace('HGMD-', 'HGMD_')
                                new_ann.append('%s.%s' % (self.resources[key], k))

                            variant[7] = '%s;%s' % (variant[7], ";".join(new_ann))
                        # add rs_ID
                        if self.resources[key] == 'dbsnp':
                            variant[2] = row[2]

                outvcf_file.writelines("\t".join(variant))
        outvcf_file.close()


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Annotate a VCF File with multiple VCFs.')

    parser.add_argument('-i', dest='vcf_file', required=True, metavar='example.vcf', help='a VCF file to be annotated')
    parser.add_argument('-r', dest='resources', required=True, metavar='1000genomes', nargs='+',
                        help='string to be added to annotated fields')
    parser.add_argument('-a', dest='ann_files', required=True, metavar='1000genomes.vcf.gz', nargs='+',
                        help='a VCF file to use for annotation')
    parser.add_argument('-n', dest='cores', required=True, metavar='4', help='number of cores to use')

    args = parser.parse_args()

    vcf_annotator = VCF_Annotator(args.vcf_file, args.ann_files, args.resources, args.cores)
    vcf_annotator.run()



#!/usr/bin/python
# -*- coding: utf-8 -*-

#script to convert vcf to csv
import os
import csv

from optparse import OptionParser

parser = OptionParser()
parser.add_option("-v", dest="vcf_file",
                  help="VCF File to Annotate", metavar="VCF")
(options, args) = parser.parse_args()

vcffile = options.vcf_file

vep_tags = 'Allele|Consequence|IMPACT|SYMBOL|Gene|Feature_type|Feature|BIOTYPE|EXON|INTRON|HGVSc|HGVSp|cDNA_position|CDS_position|Protein_position|Amino_acids|Codons|Existing_variation|DISTANCE|STRAND|FLAGS|SYMBOL_SOURCE|HGNC_ID|SIFT|PolyPhen|1000Gp3_AC|1000Gp3_AF|1000Gp3_AFR_AC|1000Gp3_AFR_AF|1000Gp3_AMR_AC|1000Gp3_AMR_AF|1000Gp3_EAS_AC|1000Gp3_EAS_AF|1000Gp3_EUR_AC|1000Gp3_EUR_AF|1000Gp3_SAS_AC|1000Gp3_SAS_AF|ALSPAC_AC|ALSPAC_AF|AltaiNeandertal|Ancestral_allele|CADD_phred|CADD_raw|CADD_raw_rankscore|DANN_rankscore|DANN_score|Denisova|ESP6500_AA_AC|ESP6500_AA_AF|ESP6500_EA_AC|ESP6500_EA_AF|Eigen-PC-phred|Eigen-PC-raw|Eigen-PC-raw_rankscore|Eigen-phred|Eigen-raw|Eigen_coding_or_noncoding|Ensembl_geneid|Ensembl_proteinid|Ensembl_transcriptid|ExAC_AC|ExAC_AF|ExAC_AFR_AC|ExAC_AFR_AF|ExAC_AMR_AC|ExAC_AMR_AF|ExAC_Adj_AC|ExAC_Adj_AF|ExAC_EAS_AC|ExAC_EAS_AF|ExAC_FIN_AC|ExAC_FIN_AF|ExAC_NFE_AC|ExAC_NFE_AF|ExAC_SAS_AC|ExAC_SAS_AF|ExAC_nonTCGA_AC|ExAC_nonTCGA_AF|ExAC_nonTCGA_AFR_AC|ExAC_nonTCGA_AFR_AF|ExAC_nonTCGA_AMR_AC|ExAC_nonTCGA_AMR_AF|ExAC_nonTCGA_Adj_AC|ExAC_nonTCGA_Adj_AF|ExAC_nonTCGA_EAS_AC|ExAC_nonTCGA_EAS_AF|ExAC_nonTCGA_FIN_AC|ExAC_nonTCGA_FIN_AF|ExAC_nonTCGA_NFE_AC|ExAC_nonTCGA_NFE_AF|ExAC_nonTCGA_SAS_AC|ExAC_nonTCGA_SAS_AF|ExAC_nonpsych_AC|ExAC_nonpsych_AF|ExAC_nonpsych_AFR_AC|ExAC_nonpsych_AFR_AF|ExAC_nonpsych_AMR_AC|ExAC_nonpsych_AMR_AF|ExAC_nonpsych_Adj_AC|ExAC_nonpsych_Adj_AF|ExAC_nonpsych_EAS_AC|ExAC_nonpsych_EAS_AF|ExAC_nonpsych_FIN_AC|ExAC_nonpsych_FIN_AF|ExAC_nonpsych_NFE_AC|ExAC_nonpsych_NFE_AF|ExAC_nonpsych_SAS_AC|ExAC_nonpsych_SAS_AF|FATHMM_converted_rankscore|FATHMM_pred|FATHMM_score|GERP++_NR|GERP++_RS|GERP++_RS_rankscore|GM12878_confidence_value|GM12878_fitCons_score|GM12878_fitCons_score_rankscore|GTEx_V6p_gene|GTEx_V6p_tissue|GenoCanyon_score|GenoCanyon_score_rankscore|H1-hESC_confidence_value|H1-hESC_fitCons_score|H1-hESC_fitCons_score_rankscore|HUVEC_confidence_value|HUVEC_fitCons_score|HUVEC_fitCons_score_rankscore|Interpro_domain|LRT_Omega|LRT_converted_rankscore|LRT_pred|LRT_score|M-CAP_pred|M-CAP_rankscore|M-CAP_score|MetaLR_pred|MetaLR_rankscore|MetaLR_score|MetaSVM_pred|MetaSVM_rankscore|MetaSVM_score|MutPred_AAchange|MutPred_Top5features|MutPred_protID|MutPred_rankscore|MutPred_score|MutationAssessor_UniprotID|MutationAssessor_pred|MutationAssessor_score|MutationAssessor_score_rankscore|MutationAssessor_variant|MutationTaster_AAE|MutationTaster_converted_rankscore|MutationTaster_model|MutationTaster_pred|MutationTaster_score|PROVEAN_converted_rankscore|PROVEAN_pred|PROVEAN_score|Polyphen2_HDIV_pred|Polyphen2_HDIV_rankscore|Polyphen2_HDIV_score|Polyphen2_HVAR_pred|Polyphen2_HVAR_rankscore|Polyphen2_HVAR_score|REVEL_rankscore|REVEL_score|Reliability_index|SIFT_converted_rankscore|SIFT_pred|SIFT_score|SiPhy_29way_logOdds|SiPhy_29way_logOdds_rankscore|SiPhy_29way_pi|TWINSUK_AC|TWINSUK_AF|Transcript_id_VEST3|Transcript_var_VEST3|Uniprot_aapos_Polyphen2|Uniprot_acc_Polyphen2|Uniprot_id_Polyphen2|VEST3_rankscore|VEST3_score|aapos|cds_strand|clinvar_clnsig|clinvar_golden_stars|clinvar_rs|clinvar_trait|codon_degeneracy|codonpos|fathmm-MKL_coding_group|fathmm-MKL_coding_pred|fathmm-MKL_coding_rankscore|fathmm-MKL_coding_score|genename|gnomAD_exomes_AC|gnomAD_exomes_AF|gnomAD_exomes_AFR_AC|gnomAD_exomes_AFR_AF|gnomAD_exomes_AFR_AN|gnomAD_exomes_AMR_AC|gnomAD_exomes_AMR_AF|gnomAD_exomes_AMR_AN|gnomAD_exomes_AN|gnomAD_exomes_ASJ_AC|gnomAD_exomes_ASJ_AF|gnomAD_exomes_ASJ_AN|gnomAD_exomes_EAS_AC|gnomAD_exomes_EAS_AF|gnomAD_exomes_EAS_AN|gnomAD_exomes_FIN_AC|gnomAD_exomes_FIN_AF|gnomAD_exomes_FIN_AN|gnomAD_exomes_NFE_AC|gnomAD_exomes_NFE_AF|gnomAD_exomes_NFE_AN|gnomAD_exomes_OTH_AC|gnomAD_exomes_OTH_AF|gnomAD_exomes_OTH_AN|gnomAD_exomes_SAS_AC|gnomAD_exomes_SAS_AF|gnomAD_exomes_SAS_AN|gnomAD_genomes_AC|gnomAD_genomes_AF|gnomAD_genomes_AFR_AC|gnomAD_genomes_AFR_AF|gnomAD_genomes_AFR_AN|gnomAD_genomes_AMR_AC|gnomAD_genomes_AMR_AF|gnomAD_genomes_AMR_AN|gnomAD_genomes_AN|gnomAD_genomes_ASJ_AC|gnomAD_genomes_ASJ_AF|gnomAD_genomes_ASJ_AN|gnomAD_genomes_EAS_AC|gnomAD_genomes_EAS_AF|gnomAD_genomes_EAS_AN|gnomAD_genomes_FIN_AC|gnomAD_genomes_FIN_AF|gnomAD_genomes_FIN_AN|gnomAD_genomes_NFE_AC|gnomAD_genomes_NFE_AF|gnomAD_genomes_NFE_AN|gnomAD_genomes_OTH_AC|gnomAD_genomes_OTH_AF|gnomAD_genomes_OTH_AN|integrated_confidence_value|integrated_fitCons_score|integrated_fitCons_score_rankscore|phastCons100way_vertebrate|phastCons100way_vertebrate_rankscore|phastCons20way_mammalian|phastCons20way_mammalian_rankscore|phyloP100way_vertebrate|phyloP100way_vertebrate_rankscore|phyloP20way_mammalian|phyloP20way_mammalian_rankscore|refcodon'.split('|')
# print(len(vep_tags))
snpeff_tags = 'Allele|Annotation|Annotation_Impact|Gene_Name|Gene_ID|Feature_Type|Feature_ID|Transcript_BioType|Rank|HGVS.c|HGVS.p|cDNA.pos/cDNA.length|CDS.pos/CDS.length|AA.pos/AA.length|Distance|ERRORS/WARNINGS/INFO'.split('|')

#Get all annotation tags from a VCF File reading all file (lazy mode!)
def get_all_info_tags(vcffile):
    tempfile = open(vcffile, 'r')
    annotation_tags = set()
    for line in tempfile:
        if not line.startswith('#'):
            variant = line.split('\t')
            string = variant[7].split(';')
            for element in string:
                element =  element.split('=')
                tag = element[0]        
                if tag not in annotation_tags:
                    annotation_tags.add(tag)
    tempfile.close()
    for item in vep_tags:
        annotation_tags.add('vep_'+item)    
    for item in snpeff_tags:
        annotation_tags.add('snpeff_'+item)    
    
    annotation_tags = sorted(annotation_tags)
    
    return annotation_tags
    
    
def parse_info_tag(string, annotation_tags):
    string = string.split(';')
    information = {}
    for element in string:
        element =  element.split('=',1)
        tag = element[0]
        if len(element) > 1:
            

            if tag == 'vep_csq':
                vep_info = element[1].split('|')
                # print(len(vep_info), vep_info)
                for i, vep_tag in enumerate(vep_tags):
                    # print(i,vep_tag)
                    information['vep_'+vep_tag] = vep_info[i]
            elif tag == 'snpeff_eff':
                snpeff_infos = element[1].split(',')
                # for snpeff_info in snpeff_infos:
                snpeff_info = snpeff_infos[0].split('|')
                for i, snpeff_tag in enumerate(snpeff_tags):
                    information['snpeff_'+snpeff_tag] = snpeff_info[i]

            else:
                information[tag] = element[1]

        else:
            information[tag] = 'Yes'
    information_list = []
    for tag in annotation_tags:
        if tag in information:
            information_list.append(str(information[tag]))
        else:
            information_list.append('')
    return information_list

def Get_vcfheader(filepath):
    vcffile = open(filepath, 'r')
    for line in vcffile:
        #print line
        if line.startswith("#CHROM"):
            header_tags = line.strip().split('\t')
        if not line.startswith("#"):
            break
    vcffile.close()
    return header_tags

vcf_header = Get_vcfheader(vcffile)

annotation_tags = get_all_info_tags(vcffile)

vcf_header = vcf_header[:7] +  annotation_tags + vcf_header[8:]

csvfilename = ".".join(os.path.basename(vcffile).split('.')[:-1])+'.csv'
csvfilepath = os.path.join(os.path.dirname(vcffile), csvfilename)

readfile = open(vcffile, 'r')
f = open(csvfilepath, "w")
csvfile = csv.writer(f, quoting=csv.QUOTE_ALL)
csvfile.writerow(vcf_header)
for line in readfile:
    if line.startswith('#CHROM'):
        vcf_header_original = line.strip().split('\t')
        vcf_header_original = vcf_header_original[:7] + list(annotation_tags) + vcf_header_original[8:]
    if not line.startswith('#'):
        variant = line.strip().split('\t')
        information = parse_info_tag(variant[7], annotation_tags)
        variant = variant[:7] + information + variant[8:]
        csvfile.writerow(variant)
        
        #new_variant = []
        ##hack to old times
        
        #print vcf_header
        #print information
        #die()
        #for tag in vcf_header:
            #tag_index = vcf_header_original.index(tag)
            #new_variant.append(variant[tag_index])
        
f.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import multiprocessing as mp
import os
import pysam
from datetime import datetime
from pynnotator import settings
from subprocess import run

class Dbnsfp(object):
    def __init__(self, vcf_file=None):
        
            self.vcf_file = vcf_file
            self.dbnfsp_header = open('%s/dbnsfp/header.vcf' % (settings.data_dir)).readlines()
            
            self.filename = os.path.splitext(os.path.basename(str(vcf_file)))[0]
            # create folder validator if it doesn't exists
            if not os.path.exists('dbnfsp'):
                os.makedirs('dbnfsp')

    def run(self):

            tstart = datetime.now()

            print(tstart, 'Starting DBNSFP annotator: ', self.vcf_file)

            command = 'bcftools annotate -a {} \
            -c "" \
             {}'.format(self.annotation, self.vcf_file)
            run(command,shell=True)

            tend = datetime.now()
            annotation_time = tend - tstart
            print(tend, 'Finished DBNSFP, it took: ', annotation_time)

if __name__ == '__main__':
    
    parser = argparse.ArgumentParser(description='Annotate a VCF File with DbNSFP.')
    parser.add_argument('-a', dest='vcf_file', required=True, metavar='example.vcf', help='a TABIX file to be annotated')
    parser.add_argument('-i', dest='vcf_file', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()
    dbnfsp = Dbnsfp(args.vcf_file)
    dbnfsp.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
from datetime import datetime
from subprocess import call, run

from pynnotator import settings

"""
- Search and Remove genotypes with [0/0, ./.]
- Search and Replace chr from the beggining of the chromossome names
- Sort VCF by 1...22, X, Y, MT
- Remove previous snpeff annotations from VCF
- Remove annotation sumGLbyD from VCF that causes imcompatibility with snpeff
"""

toolname = 'sanity_check'

# enable perl5_lib
env = os.environ.copy()
env['PERL5LIB'] = settings.vcftools_dir_perl


class Sanity_check(object):
    def __init__(self, vcf_file=None):

        self.vcf_file = vcf_file
        self.filename = os.path.splitext(os.path.basename(str(vcf_file)))[0]

        # create folder sanity_check if it doesn't exists
        if not os.path.exists('sanity_check'):
            os.makedirs('sanity_check')
            # enter inside folder
            # os.chdir('sanity_check')

    def run(self):

        tstart = datetime.now()
        print(tstart, 'Starting sanity_check: ', self.vcf_file)

        self.check()

        tend = datetime.now()
        annotation_time = tend - tstart
        print(tend, 'Finished sanity_check, it took: ', annotation_time)

        # sanity vcf file with Vcftools

    def check(self):

        command = 'vcf-sort {} > sanity_check/sorted.vcf'.format(self.vcf_file)
        run(command, shell=True)
        # file_vcf = open("%s" % (self.vcf_file), 'r')
        # out_vcf = open('sanity_check/onlyvariants.vcf', 'w')
        # for line in file_vcf:
        #     if line.startswith('#'):
        #         # fix varscan vcfs            
        #         line = line.replace(')>', '>')
        #         if not line.startswith('##INFO=<ID=EFF'):
        #             if not line.startswith('##INFO=<ID=sumGLbyD'):
        #                 if not line.startswith('##INFO=<ID=CSQ'):
        #                     out_vcf.writelines(line)
        #     else:
        #         row = line.split('\t')

        #         # removing EFF INFO from VCF
        #         INFO = row[7]
        #         NEWINFO = []
        #         for info in INFO.split(";"):
        #             # removing snpeff from annotations
        #             if "EFF=" in info:
        #                 pass
        #             # ugly hack to remove this field, because of some bugs in GATK/snpeff that require tyou to have a description about all fields
        #             elif "sumGLbyD=" in info:
        #                 pass
        #             elif "CSQ=" in info:
        #                 pass
        #             else:
        #                 NEWINFO.append(info)
        #         row[7] = ";".join(NEWINFO)
        #         row[2] = '.'
        #         # remove info field from vcf
        #         # row[7] = ''
        #         genotype = row[-1].strip().split(':')[0]
        #         # remove chr from the beggining of chromossome name
        #         row[0] = row[0].replace('chr', '')
        #         # remove genotypes with 0/0
        #         forbidden = ['0/0', './.']
        #         allowed_chr = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16',
        #                        '17', '18', '19', '20', '21', '22', 'X', 'Y', 'M', 'MT']
        #         if genotype not in forbidden:
        #             if row[0] in allowed_chr:
        #                 out_vcf.writelines('\t'.join(row))

        # out_vcf.close()
        # file_vcf.close()

        # remove EFF Field
        # vcf_tools_dir = '/lgc/programs/vcftools_0.1.10/bin'
        # os.environ["PERL5LIB"] = "/lgc/programs/vcftools_0.1.10/lib/perl5/site_perl/"

        # command = '%s/vcf-annotate onlyvariants.vcf -r INFO/EFF > withouteff.vcf' % (vcf_tools_dir)
        # os.system(command)

        # sort VCF
        # logging.info('Starting Sort VCF')
        # get header

        # command = "grep '^#' sanity_check/onlyvariants.vcf > sanity_check/checked.vcf"
        # call(command, shell=True)

        # # only chromossome numbers first

        # command = "grep -E -v '^X|^Y|^M|^#|^GL' sanity_check/onlyvariants.vcf | sort -n -k1 -k2 >> sanity_check/checked.vcf"
        # call(command, shell=True)

        # # only X
        # command = "grep -E '^X' sanity_check/onlyvariants.vcf | sort -k1,1d -k2,2n >> sanity_check/checked.vcf"
        # call(command, shell=True)

        # # only Y
        # command = "grep -E '^Y' sanity_check/onlyvariants.vcf | sort -k1,1d -k2,2n >> sanity_check/checked.vcf"
        # call(command, shell=True)

        # # only MT
        # command = "grep -E '^M' sanity_check/onlyvariants.vcf | sort -k1,1d -k2,2n >> sanity_check/checked.vcf"
        # call(command, shell=True)

        # command = 'rm sanity_check/onlyvariants.vcf'
        # run(command, shell=True)


        # p = subprocess.call(command, 
        #     cwd=os.getcwd(), 
        #     env=env, 
        #     shell=True)

        time_end = datetime.now()
        # print(time_end, 'This vcf was sucessfully checked with sanity_check')

        # if p == 0:
        #     print 'This vcf was validated by vcf-sanity_check'
        # else:
        #     print 'Sorry this vcf could not be annotated by %s' % (toolname)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Sanity Check a VCf File')

    parser.add_argument('-i', dest='vcf_file', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()

    sanity_check = Sanity_check(args.vcf_file)
    sanity_check.run()



import os

###MAIN SETTINGS

snpEff_memory = "4G"
snpsift_memory = "4G"
snpsift_merge_memory = "4G"
vep_cores = 4
vcf_annotator_cores = 4
dbnsfp_cores = 4

# LIBS SETTINGS
BASE_DIR = os.path.dirname(__file__)
libs_dir = os.path.join(BASE_DIR, 'libs')
data_dir = os.path.join(BASE_DIR, 'data')

###LIBS

# htslib (tabix)
htslib_version = '1.8'
htslib_file = 'htslib-%s.tar.bz2' % (htslib_version)
htslib_source = 'https://github.com/samtools/htslib/releases/download/%s/%s' % (htslib_version, htslib_file)
htslib_dir = '%s/htslib/htslib-%s' % (libs_dir, htslib_version)

vcfanno_dir = '%s/vcfanno/' % (libs_dir)

# vcftools
vcftools_version = '0.1.15'
vcftools_file = 'vcftools-%s.tar.gz' % (vcftools_version)
vcftools_source = 'https://github.com/vcftools/vcftools/releases/download/v%s/vcftools-%s.tar.gz' % (
    vcftools_version, vcftools_version)
# validation
vcftools_dir = "%s/vcftools/vcftools-%s/src/cpp" % (libs_dir, vcftools_version)
vcftools_dir_perl = "%s/vcftools/vcftools-%s/src/perl" % (libs_dir, vcftools_version)
vcf_validator_dir = "%s/vcf-validator" % (libs_dir)

# snpeff
snpeff_database = 'GRCh37.75'  # this is the last build from GRCh37
snpeff_version = 'snpEff_latest_core'
snpeff_version = 'snpEff_v4_3t_core'  # need to test version 4.3
snpeff_source = 'http://sourceforge.net/projects/snpeff/files/%s.zip' % (snpeff_version)

snpeff_dir = os.path.join(libs_dir, 'snpeff', 'snpEff')
snpeff_data_dir = os.path.join(data_dir, 'snpeff_data')

# vep
vep_release = '92.4'
vep_source = 'https://github.com/Ensembl/ensembl-vep/archive/release/%s.zip' % (vep_release)
vep_cache_dir = os.path.join(data_dir, 'vep_data')
vep_dir = '%s/vep/src/ensembl-vep/' % (libs_dir)

# gemini
gemini_file = 'gemini_install.py'
gemini_source = 'https://github.com/arq5x/gemini/raw/master/gemini/scripts/%s' % (gemini_file)

###Datasets

# Decipher
hi_predictions_file = 'HI_Predictions_Version3.bed.gz'
hi_predictions_source = 'https://decipher.sanger.ac.uk/files/downloads/%s' % (hi_predictions_file)
hi_predictions = '%s/decipher/%s' % (data_dir, hi_predictions_file)

population_cnv_file = 'population_cnv.txt.gz'
population_cnv_source = 'https://decipher.sanger.ac.uk/files/downloads/%s' % (population_cnv_file)

ddg2p_file = 'DDG2P.csv.gz'
ddg2p_source = 'http://www.ebi.ac.uk/gene2phenotype/downloads/%s' % (ddg2p_file)

# ensembl #HGMD PUBLIC
vep_major_release = '92'
ensembl_phenotype_file = 'homo_sapiens_phenotype_associated.vcf.gz'
ensembl_phenotype_source = 'ftp://ftp.ensembl.org/pub/grch37/release-%s/variation/vcf/homo_sapiens/%s' % (
    vep_major_release, ensembl_phenotype_file)

ensembl_clinically_file = 'homo_sapiens_clinically_associated.vcf.gz'
ensembl_clinically_source = 'ftp://ftp.ensembl.org/pub/grch37/release-%s/variation/vcf/homo_sapiens/%s' % (
    vep_major_release, ensembl_clinically_file)

# 1000genomes
genomes1k_vcf = 'ALL.wgs.phase3_shapeit2_mvncall_integrated_v5b.20130502.sites.vcf'
genomes1k_file = '%s.gz' % (genomes1k_vcf)
genomes1k_source = 'ftp://ftp.1000genomes.ebi.ac.uk/vol1/ftp/release/20130502/%s' % (genomes1k_file)

# dbsnp
dbsnp_file = 'All_20180423.vcf.gz'
dbsnp_source = 'ftp://ftp.ncbi.nih.gov/snp/organisms/human_9606_b151_GRCh37p13/VCF/%s' % (dbsnp_file)
dbsnp = '%s/dbsnp/%s' % (data_dir, dbsnp_file)

clinvar_file = 'clinvar_20180429.vcf.gz'
clinvar_source = 'ftp://ftp.ncbi.nlm.nih.gov/pub/clinvar/vcf_GRCh37/%s' % (clinvar_file)

# ESP
esp_basename = 'ESP6500SI-V2-SSA137.GRCh38-liftover'
esp_file = '%s.snps_indels.vcf.tar.gz' % (esp_basename)
esp_source = 'http://evs.gs.washington.edu/evs_bulk_data/%s' % (esp_file)
esp_final_file = 'esp6500si.vcf.gz'

# dbnsfp
dbnsfp_version = '3.5a'
dbnsfp_file = 'dbNSFP%s.txt.gz' % (dbnsfp_version)
dbnsfp_link = 'ftp://dbnsfp:dbnsfp@dbnsfp.softgenetics.com/dbNSFPv%s.zip' % (dbnsfp_version)
dbnsfp = '%s/dbnsfp/dbNSFP%s.txt.gz' % (data_dir, dbnsfp_version)

# dbscsnv
dbscsnv_version = '1.1'
dbscsnv_file = 'dbscSNV%s.txt.gz' % (dbscsnv_version)
dbscsnv_source = 'ftp://dbnsfp:dbnsfp@dbnsfp.softgenetics.com/dbscSNV%s.zip' % (dbscsnv_version)
dbscsnv = '%s/dbnsfp/dbscSNV%s.txt.gz' % (data_dir, dbscsnv_version)

data_file = "pynnotator-data.latest.tar"
data_source = "https://mendelmd.org/%s" % (data_file)

libs_file = "pynnotator-libs.latest.tar.gz"
libs_source = "https://mendelmd.org/%s" % (libs_file)

#hgmd
hgmd = "HGMD_PRO_2017.3_hg19.vcf.gz"
hgmd_file = '%s/hgmd/%s' % (data_dir, hgmd)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
import pysam
from collections import OrderedDict
from datetime import datetime

from pynnotator import settings
from subprocess import run
toolname = 'merge'


class Merge(object):
    def __init__(self, vcffile=None):

        self.vcffile = vcffile

        self.filename = os.path.splitext(os.path.basename(str(vcffile)))[0]

        self.basedir = os.getcwd()

        # create folder merge if it doesn't exists
        if not os.path.exists('merge'):
            os.makedirs('merge')
        # enter inside folder
        os.chdir('merge')

    def run(self):
        tstart = datetime.now()
        print(tstart, 'Starting merge: ', self.vcffile)

        self.merge()

        tend = datetime.now()
        annotation_time = tend - tstart
        print(tend, 'Finished merge, it took: ', annotation_time)

        # merge VCF Files with VariantAnnotator

    def merge(self):

        # print(BASE_DIR)

        files = ['../snpeff/snpeff.output.vcf', '../vep/vep.output.sorted.vcf', '../snpsift/snpsift.final.vcf']
        for file in files:
            command = 'bgzip {}'.format(file)
            run(command, shell=True)
            command = 'tabix -p vcf {}.gz'.format(file)
            run(command, shell=True)


        config = open('config.toml', 'w')
        config.write("""[[annotation]]
file="{}/snpsift/snpsift.final.vcf.gz"
fields = ["VARTYPE", "SNP", "MNP", "INS", "DEL", "MIXED", "HOM", "HET"]
ops=["first", "first", "first", "first", "first", "first", "first", "first"]

[[annotation]]
file="{}/dbsnp/{}"
fields = ["ID", "RS", "RSPOS", "RV", "VP", "GENEINFO", "dbSNPBuildID", "SAO", "SSR", "WGT", "VC", "PM", "TPA", "PMC", "S3D", "SLO", "NSF", "NSM", "NSN", "REF", "SYN", "U3", "U5", "ASS", "DSS", "INT", "R3", "R5", "OTH", "CFL", "ASP", "MUT", "VLD", "G5A", "G5", "HD", "GNO", "KGPhase1", "KGPhase3", "CDA", "LSD", "MTP", "OM", "NOC", "WTD", "NOV", "NC", "CAF", "COMMON", "TOPMED"]
ops=["first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first"]

[[annotation]]
file="{}/dbsnp/{}"
fields = ["AF_ESP", "AF_EXAC", "AF_TGP", "ALLELEID", "CLNDN", "CLNDNINCL", "CLNDISDB", "CLNDISDBINCL", "CLNHGVS", "CLNREVSTAT", "CLNSIG", "CLNSIGCONF", "CLNSIGINCL", "CLNVC", "CLNVCSO", "CLNVI", "DBVARID", "GENEINFO", "MC", "ORIGIN", "RS", "SSR", ]
ops=["first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first"]

[[annotation]]
file="{}/1000genomes/ALL.wgs.phase3_shapeit2_mvncall_integrated_v5b.20130502.sites.vcf.gz"
fields = ["CIEND", "CIPOS", "CS", "END", "IMPRECISE", "MC", "MEINFO", "MEND", "MLEN", "MSTART", "SVLEN", "SVTYPE", "TSD", "AC", "AF", "NS", "AN", "EAS_AF", "EUR_AF", "AFR_AF", "AMR_AF", "SAS_AF", "DP", "AA", "VT", "EX_TARGET", "MULTI_ALLELIC", "OLD_VARIANT"]
ops=["first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first", "first"]

[[annotation]]
file="{}/decipher/HI_Predictions_Version3.bed.gz"
names=["HI_PREDICTION"]
columns=[4]
ops=["uniq"]

[[annotation]]
file="{}/ensembl/homo_sapiens_clinically_associated.vcf.gz"
names = ["ensembl_clin_ClinVar_201706", "ensembl_clin_dbSNP_150", "ensembl_clin_HGMD", "ensembl_clin_PhenCode", "ensembl_clin_COSMIC_81", "ensembl_clin_ESP_20141103", "ensembl_clin_TSA", "ensembl_clin_E_Cited", "ensembl_clin_E_Multiple_observations", "ensembl_clin_E_Freq", "ensembl_clin_E_Hapmap", "ensembl_clin_E_Phenotype_or_Disease", "ensembl_clin_E_ESP", "ensembl_clin_E_1000G", "ensembl_clin_E_ExAC", "ensembl_clin_CLIN_risk_factor", "ensembl_clin_CLIN_protective", "ensembl_clin_CLIN_confers_sensitivity", "ensembl_clin_CLIN_other", "ensembl_clin_CLIN_drug_response", "ensembl_clin_CLIN_uncertain_significance", "ensembl_clin_CLIN_benign", "ensembl_clin_CLIN_likely_pathogenic", "ensembl_clin_CLIN_pathogenic", "ensembl_clin_CLIN_likely_benign", "ensembl_clin_CLIN_histocompatibility", "ensembl_clin_CLIN_not_provided", "ensembl_clin_CLIN_association", "ensembl_clin_MA", "ensembl_clin_MAF", "ensembl_clin_MAC", "ensembl_clin_AA"]
fields = ["ClinVar_201706", "dbSNP_150", "HGMD", "PhenCode", "COSMIC_81", "ESP_20141103", "TSA", "E_Cited", "E_Multiple_observations", "E_Freq", "E_Hapmap", "E_Phenotype_or_Disease", "E_ESP", "E_1000G", "E_ExAC", "CLIN_risk_factor", "CLIN_protective", "CLIN_confers_sensitivity", "CLIN_other", "CLIN_drug_response", "CLIN_uncertain_significance", "CLIN_benign", "CLIN_likely_pathogenic", "CLIN_pathogenic", "CLIN_likely_benign", "CLIN_histocompatibility", "CLIN_not_provided", "CLIN_association", "MA", "MAF", "MAC", "AA"]
ops=["first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first"]

[[annotation]]
file="{}/ensembl/homo_sapiens_phenotype_associated.vcf.gz"
names = ["ensembl_phen_ClinVar_201706","ensembl_phen_dbSNP_150","ensembl_phen_HGMD","ensembl_phen_PhenCode","ensembl_phen_COSMIC_81","ensembl_phen_ESP_20141103","ensembl_phen_TSA","ensembl_phen_E_Cited","ensembl_phen_E_Multiple_observations","ensembl_phen_E_Freq","ensembl_phen_E_Hapmap","ensembl_phen_E_Phenotype_or_Disease","ensembl_phen_E_ESP","ensembl_phen_E_1000G","ensembl_phen_E_ExAC","ensembl_phen_CLIN_risk_factor","ensembl_phen_CLIN_protective","ensembl_phen_CLIN_confers_sensitivity","ensembl_phen_CLIN_other","ensembl_phen_CLIN_drug_response","ensembl_phen_CLIN_uncertain_significance","ensembl_phen_CLIN_benign","ensembl_phen_CLIN_likely_pathogenic","ensembl_phen_CLIN_pathogenic","ensembl_phen_CLIN_likely_benign","ensembl_phen_CLIN_histocompatibility","ensembl_phen_CLIN_not_provided","ensembl_phen_CLIN_association","ensembl_phen_MA","ensembl_phen_MAF","ensembl_phen_MAC","ensembl_phen_AA"]
fields = ["ClinVar_201706", "dbSNP_150", "HGMD", "PhenCode", "COSMIC_81", "ESP_20141103", "TSA", "E_Cited", "E_Multiple_observations", "E_Freq", "E_Hapmap", "E_Phenotype_or_Disease", "E_ESP", "E_1000G", "E_ExAC", "CLIN_risk_factor", "CLIN_protective", "CLIN_confers_sensitivity", "CLIN_other", "CLIN_drug_response", "CLIN_uncertain_significance", "CLIN_benign", "CLIN_likely_pathogenic", "CLIN_pathogenic", "CLIN_likely_benign", "CLIN_histocompatibility", "CLIN_not_provided", "CLIN_association", "MA", "MAF", "MAC", "AA"]
ops=["first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first"]

[[annotation]]
file="{}/gnomead/gnomad.exomes.r2.0.2.sites.vcf.bgz"
fields = ["AC", "AF", "AN", "BaseQRankSum", "ClippingRankSum", "DB", "DP", "FS", "InbreedingCoeff", "MQ", "MQRankSum", "QD", "ReadPosRankSum", "SOR", "VQSLOD", "VQSR_culprit", "VQSR_NEGATIVE_TRAIN_SITE", "VQSR_POSITIVE_TRAIN_SITE", "GQ_HIST_ALT", "DP_HIST_ALT", "AB_HIST_ALT", "GQ_HIST_ALL", "DP_HIST_ALL", "AB_HIST_ALL", "AC_AFR", "AC_AMR", "AC_ASJ", "AC_EAS", "AC_FIN", "AC_NFE", "AC_OTH", "AC_SAS", "AC_Male", "AC_Female", "AN_AFR", "AN_AMR", "AN_ASJ", "AN_EAS", "AN_FIN", "AN_NFE", "AN_OTH", "AN_SAS", "AN_Male", "AN_Female", "AF_AFR", "AF_AMR", "AF_ASJ", "AF_EAS", "AF_FIN", "AF_NFE", "AF_OTH", "AF_SAS", "AF_Male", "AF_Female", "GC_AFR", "GC_AMR", "GC_ASJ", "GC_EAS", "GC_FIN", "GC_NFE", "GC_OTH", "GC_SAS", "GC_Male", "GC_Female", "AC_raw", "AN_raw", "AF_raw", "GC_raw", "GC", "Hom_AFR", "Hom_AMR", "Hom_ASJ", "Hom_EAS", "Hom_FIN", "Hom_NFE", "Hom_OTH", "Hom_SAS", "Hom_Male", "Hom_Female", "Hom_raw", "Hom", "STAR_AC", "STAR_AC_raw", "STAR_Hom", "POPMAX", "AC_POPMAX", "AN_POPMAX", "AF_POPMAX", "DP_MEDIAN", "DREF_MEDIAN", "GQ_MEDIAN", "AB_MEDIAN", "AS_RF", "AS_FilterStatus", "AS_RF_POSITIVE_TRAIN", "AS_RF_NEGATIVE_TRAIN", "CSQ", "AN_FIN_Male", "AN_EAS_Female", "AN_NFE_Female", "AC_AFR_Male", "AN_AMR_Female", "AF_AMR_Male", "Hemi_NFE", "Hemi_AFR", "AC_ASJ_Female", "AF_FIN_Female", "AN_ASJ_Male", "AC_OTH_Female", "GC_OTH_Male", "GC_FIN_Male", "AC_NFE_Female", "AC_EAS_Male", "AC_OTH_Male", "GC_SAS_Male", "Hemi_AMR", "AC_NFE_Male", "Hemi", "AN_FIN_Female", "GC_EAS_Male", "GC_ASJ_Female", "GC_SAS_Female", "GC_ASJ_Male", "Hemi_SAS", "AN_ASJ_Female", "AF_FIN_Male", "AN_OTH_Male", "AF_AFR_Male", "STAR_Hemi", "AF_SAS_Male", "Hemi_ASJ", "AN_SAS_Female", "AN_AFR_Female", "Hemi_raw", "AF_OTH_Male", "AC_SAS_Female", "AF_NFE_Female", "AF_EAS_Female", "AN_OTH_Female", "AF_EAS_Male", "AF_SAS_Female", "GC_AFR_Female", "AF_AFR_Female", "AC_FIN_Female", "Hemi_OTH", "GC_AMR_Male", "AC_AFR_Female", "GC_NFE_Male", "AF_AMR_Female", "GC_NFE_Female", "AN_AFR_Male", "AN_NFE_Male", "AC_AMR_Male", "GC_AMR_Female", "AC_SAS_Male", "AF_ASJ_Male", "GC_FIN_Female", "AC_EAS_Female", "AC_AMR_Female", "Hemi_FIN", "AC_FIN_Male", "GC_EAS_Female", "AF_ASJ_Female", "AF_OTH_Female", "GC_AFR_Male", "AN_SAS_Male", "AF_NFE_Male", "AN_EAS_Male", "AC_ASJ_Male", "Hemi_EAS", "AN_AMR_Male", "GC_OTH_Female", "segdup", "lcr"]
ops=["first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first","first"]

[[annotation]]
file="{}/snpeff/snpeff.output.vcf.gz"
fields = ["ANN"]
names = ["snpeff_eff"]
ops=["first"]

[[annotation]]
file="{}/vep/vep.output.sorted.vcf.gz"
fields = ["CSQ"]
names = ["vep_csq"]
ops=["first"]

            """.format(self.basedir, settings.data_dir, settings.dbsnp_file, settings.data_dir, settings.clinvar_file, settings.data_dir, settings.data_dir,settings.data_dir,settings.data_dir,settings.data_dir,self.basedir, self.basedir))
#settings.data_dir,         
#[[annotation]]
#file="{}/hgmd/HGMD_PRO_2017.3_hg19.vcf.gz"
#fields = ["CLASS", "MUT", "GENE", "STRAND", "DNA", "PROT", "DB", "PHEN"]
#ops=["first", "first", "first", "first", "first", "first", "first", "first"]
        
        config.close()
        command = '{}/vcfanno/vcfanno_linux64 -p 16 config.toml ../{} > ../annotation.final.vcf'.format(settings.libs_dir, self.vcffile)
        run(command,shell=True)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='MERGE all VCF Files from previous methods.')
    parser.add_argument('-i', dest='vcffile', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()

    merge = Merge(args.vcffile)
    merge.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
import subprocess
from datetime import datetime
from subprocess import call

from pynnotator import settings

toolname = 'hgmd'


class HGMD(object):
    def __init__(self, vcffile=None):

        self.vcffile = vcffile

        self.filename = os.path.splitext(os.path.basename(str(vcffile)))[0]

        # create folder vep if it doesn't exists
        if not os.path.exists('hgmd'):
            os.makedirs('hgmd')
            # enter inside folder
            # os.chdir('vep')

    def run(self):
        tstart = datetime.now()
        print(tstart, 'Starting hgmd annotation: ', self.vcffile)

        self.annotate()

        tend = datetime.now()
        annotation_time = tend - tstart
        print(tend, 'Finished hgmd annotation, it took: ', annotation_time)

    # convert and annotate the vcf file to vep
    def annotate(self):

        if self.vcffile.endswith('.gz'):
            command = 'cp %s hgmd/hgmd.raw.vcf.gz' % (self.vcffile)
        else:
            command = 'bgzip -c %s > hgmd/hgmd.raw.vcf.gz' % (self.vcffile)
        
        call(command, shell=True)

        command = 'tabix -p vcf hgmd/hgmd.raw.vcf.gz'
        call(command, shell=True)

        command = '''bcftools annotate -a %s/hgmd/%s -c INFO hgmd/hgmd.raw.vcf.gz > hgmd/hgmd.vcf
        ''' % (settings.data_dir, settings.hgmd)

        command = '''zcat hgmd/hgmd.raw.vcf.gz | %s/vcf-annotate \
        -a %s \
        --fill-type \
        -d key=INFO,ID=CLASS,Number=1,Type=String,Description=\"Mutation Category, https://portal.biobase-international.com/hgmd/pro/global.php#cats\" \
        -d key=INFO,ID=MUT,Number=1,Type=String,Description=\"HGMD mutant allele\" \
        -d key=INFO,ID=GENE,Number=1,Type=String,Description=\"Gene symbol\" \
        -d key=INFO,ID=STRAND,Number=1,Type=String,Description=\"Gene strand\" \
        -d key=INFO,ID=DNA,Number=1,Type=String,Description=\"DNA annotation\" \
        -d key=INFO,ID=PROT,Number=1,Type=String,Description=\"Protein annotation\" \
        -d key=INFO,ID=DB,Number=1,Type=String,Description=\"dbSNP identifier, build 146\" \
        -d key=INFO,ID=PHEN,Number=1,Type=String,Description=\"HGMD primary phenotype\" \
        -c CHROM,FROM,TO,INFO/CLASS,INFO/MUT,INFO/GENE,INFO/STRAND,INFO/DNA,INFO/PROT,INFO/DB,INFO/PHEN > hgmd/hgmd.vcf 2>hgmd/hgmd.errors.txt \
        ''' % (settings.vcftools_dir_perl, settings.hgmd_file)

        # print(command)
        call(command, shell=True)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Annotate a VCF File with HGMD.')
    parser.add_argument('-i', dest='vcffile', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()

    hgmd = HGMD(args.vcffile)
    hgmd.run()



#!/usr/bin/python

import os
import platform
import subprocess
from subprocess import call

from . import settings

BASE_DIR = os.path.dirname(__file__)
libs_dir = os.path.join(BASE_DIR, 'libs')
data_dir = os.path.join(BASE_DIR, 'data')


class Installer(object):
    """A class to annotate VCFs

    Attributes:
        vcf_file: a VCF file to be annotated
    """

    def __init__(self):
        """Return a Pynnotator Installer object """

    def install(self):

        self.install_requirements()        
        #self.download_libs()
        self.download_data()
        self.install_libs()
        self.build_datasets()

    def install_requirements(self):
        """Install Ubuntu Requirements"""
        print('Installing Requirements')
        print(platform.dist())
        if platform.dist()[0] in ['Ubuntu', 'LinuxMint']:
            command = 'sudo apt-get install -y apache2 bcftools build-essential cpanminus curl git libbz2-dev libcurl4-openssl-dev liblocal-lib-perl liblzma-dev libmysqlclient-dev libpng-dev libpq-dev libssl-dev manpages mysql-client openssl perl perl-base pkg-config python3-dev python3-pip python3-setuptools sed tabix unzip vcftools vim wget zlib1g-dev apache2 build-essential cpanminus curl git libmysqlclient-dev libpng-dev libssl-dev locales manpages mysql-client openssl perl perl-base unzip vim wget'  # lamp-server^
            sts = call(command, shell=True)

            try:
                subprocess.call(['java', '-version'])
            except:

                command = """sudo apt install -y software-properties-common
                       sudo add-apt-repository -y ppa:webupd8team/java
                       sudo apt-get update
                       echo "oracle-java8-installer shared/accepted-oracle-license-v1-1 select true" | sudo debconf-set-selections
                       sudo apt-get -y install oracle-java8-installer"""

                sts = call(command, shell=True)
        elif platform.dist()[0] in ['debian']:
            
            command = 'sudo apt-get install -y libmodule-install-perl apache2 bcftools build-essential cpanminus curl git libbz2-dev libcurl4-openssl-dev liblocal-lib-perl liblzma-dev default-libmysqlclient-dev libpng-dev libpq-dev libssl-dev manpages mysql-client openssl perl perl-base pkg-config python3-dev python3-pip python3-setuptools sed tabix unzip vcftools vim wget zlib1g-dev apache2 build-essential cpanminus curl git libpng-dev libssl-dev locales manpages mysql-client openssl perl perl-base unzip vim wget'  # lamp-server^
            sts = call(command, shell=True)
            command = 'sudo apt-get install default-jre default-jdk'
            sts = call(command, shell=True)
        
        elif platform.dist()[0] in ['redhat', 'centos']:

            command = 'sudo yum install libcurl-devel sed vcftools bcftools tabix zlib-devel postgresql96-libs perl-local-lib perl-App-cpanminus curl unzip wget'
            sts = call(command, shell=True)
            command = """sudo yum groupinstall 'Development Tools'"""
            sts = call(command, shell=True)
            command = """sudo yum install gcc gcc-c++ make openssl-devel"""
            sts = call(command, shell=True)

            try:
                subprocess.call(['java', '-version'])
            except:
                command = """wget http://javadl.oracle.com/webapps/download/AutoDL?BundleId=225342_090f390dda5b47b9b721c7dfaa008135 -o java.rpm"""
                sts = call(command, shell=True)
                command = """sudo rpm -i java.rpm"""
                sts = call(command, shell=True)

        # Perl Requirements
        command = "sudo cpanm DBI DBD::mysql File::Copy::Recursive Archive::Extract Archive::Zip LWP::Simple Bio::Root::Version LWP::Protocol::https Bio::DB::Fasta CGI Test::utf8 Test::File inc::Module::Install"
        sts = call(command, shell=True)
        command = "cpanm --local-lib=~/perl5 local::lib && eval $(perl -I ~/perl5/lib/perl5/ -Mlocal::lib)"
        sts = call(command, shell=True)
    def install_libs(self):

        if not os.path.exists(libs_dir):
            os.makedirs(libs_dir)
        os.chdir(libs_dir)

        # self.install_java()

        self.install_htslib()
        self.install_vcftools()
        self.install_snpeff()
        self.install_gemini()
        self.install_vep()
        self.install_vcf_anno()

    def build_datasets(self):
        print("Building Datasets")

        if not os.path.exists(data_dir):
            os.makedirs(data_dir)
        os.chdir(data_dir)

        self.download_snpeff_data()
        self.download_vep_data()

        self.download_decipher()
        self.download_ensembl()
        self.download_1000genomes()
        self.download_dbsnp()
        self.download_esp()
        self.download_dbnsfp()

    def download_data(self):
        print("Downloading Data")

        os.chdir(settings.BASE_DIR)

        if not os.path.exists(settings.data_dir):
            command = "wget %s -O %s" % (settings.data_source, settings.data_file)
            call(command, shell=True)

            print("Extracting Data...")
            command = "tar -xvf %s" % (settings.data_file)
            call(command, shell=True)

            print("Deleting Compressed File...")
            command = "rm %s" % (settings.data_file)
            call(command, shell=True)

    def download_libs(self):
        print("Downloading Libs")

        os.chdir(settings.BASE_DIR)

        if not os.path.exists(settings.libs_dir):
            command = "wget %s -O %s" % (settings.libs_source, settings.libs_file)
            call(command, shell=True)

            print("Extracting Data...")
            command = "tar -zxvf %s" % (settings.libs_file)
            call(command, shell=True)

            print("Deleting Compressed File...")
            command = "rm %s" % (settings.libs_file)
            # call(command, shell=True)

    def install_java(self):
        if not os.path.isdir('java'):
            call('mkdir java', shell=True)
        os.chdir('java')

        # check if file exists
        if not os.path.isfile(java_file):
            command = """
            wget -c %s -O %s
            tar -zxvf %s
            """ % (settings.java_source, settings.java_file, settings.java_file)
            call(command, shell=True)

        os.chdir(libs_dir)

    def install_htslib(self):
        if not os.path.isdir('htslib'):
            call('mkdir htslib', shell=True)

        os.chdir('htslib')
        # check if file exists
        if not os.path.isfile(settings.htslib_file):
            command = """
            wget -c %s
            tar -jxvf %s
            cd htslib-%s; ./configure; make
            """ % (settings.htslib_source, settings.htslib_file, settings.htslib_version)

            call(command, shell=True)

        os.chdir(libs_dir)

    def install_vcftools(self):
        if not os.path.isdir('vcftools'):
            call('mkdir vcftools', shell=True)
        os.chdir('vcftools')
        # check if file exists
        if not os.path.isfile(settings.vcftools_file):
            command = """
            wget -c %s
            tar -zxvf %s
            cd vcftools-%s
            ./configure; make
            """ % (settings.vcftools_source, settings.vcftools_file, settings.vcftools_version)
            call(command, shell=True)

        os.chdir(libs_dir)

    def install_snpeff(self):
        if not os.path.isdir('snpeff'):
            call('mkdir snpeff', shell=True)
        os.chdir('snpeff')
        # check if file exists
        if not os.path.isfile('%s.zip' % (settings.snpeff_version)):
            command = """
            wget -c %s
            unzip %s.zip
            """ % (settings.snpeff_source, settings.snpeff_version)
            call(command, shell=True)

            # change data_dir
            # from ./data/ to data.dir = ../../../data/snpeff_data
            command = """
            sed -i 's/\.\/data\//\.\.\/\.\.\/\.\.\/data\/snpeff_data/g' snpEff/snpEff.config"""
            call(command, shell=True)

        os.chdir(libs_dir)

    def install_gemini(self):
        if not os.path.isdir('gemini'):
            call('mkdir gemini', shell=True)
        os.chdir('gemini')
        # check if file exists
        if not os.path.isfile(settings.gemini_file):
            command = 'wget -c %s -O %s' % (settings.gemini_source, settings.gemini_file)
            call(command, shell=True)

        os.chdir(libs_dir)

    def install_vep(self):

        os.chdir(libs_dir)
        if not os.path.exists('vep'):
            os.makedirs('vep')
            os.chdir('vep')
            command = 'bash {}/scripts/install_vep.sh'.format(settings.BASE_DIR)
            call(command, shell=True)
    def install_vcf_anno(self):

        os.chdir(libs_dir)
        if not os.path.exists('vcfanno'):
            os.makedirs('vcfanno')
            os.chdir('vcfanno')
            command = 'wget https://github.com/brentp/vcfanno/releases/download/v0.2.9/vcfanno_linux64'
            call(command, shell=True)
            command = 'chmod +x vcfanno_linux64'
            call(command, shell=True)
        
        # if not os.path.isfile('%s.zip' % (settings.vep_release)):
        #     command = """
        #     wget %s -O %s.zip
        #     unzip %s.zip
        #     """ % (settings.vep_source, settings.vep_release, settings.vep_release)
        #     call(command, shell=True)
        #
        #     os.chdir(settings.vep_dir)
            # download vep cache
            # command = """perl INSTALL.pl -a a --NO_TEST"""

        os.chdir(libs_dir)

    def download_snpeff_data(self):

        # download snpeff data
        if not os.path.isdir(os.path.join(settings.snpeff_data_dir, settings.snpeff_database)):
            command = "java -jar %s/snpEff.jar download -c %s/snpEff.config -v %s" % (
                settings.snpeff_dir, settings.snpeff_dir, settings.snpeff_database)
            call(command, shell=True)

    def download_vep_data(self):

        if not os.path.isdir(settings.vep_dir):
            command = 'mkdir -p {}'.format(settings.vep_dir)
            call(command, shell=True)

        os.chdir(settings.vep_dir)
        # download vep cache
        if not os.path.isdir(settings.vep_cache_dir):
            command = """perl ./INSTALL.pl -a acf -s homo_sapiens -c %s --ASSEMBLY GRCh37 --NO_TEST
            """ % (settings.vep_cache_dir)
            call(command, shell=True)

        if not os.path.isdir('{}/Plugins'.format(settings.vep_cache_dir)):
            command = 'mkdir -p {}/Plugins'.format(settings.vep_cache_dir)
            call(command, shell=True)
        if not os.path.isfile('{}/Plugins/dbNSFP.pm'.format(settings.vep_cache_dir)):
            command = 'wget -O {}/Plugins/dbNSFP.pm https://github.com/Ensembl/VEP_plugins/raw/release/92/dbNSFP.pm'.format(settings.vep_cache_dir)
            call(command, shell=True)
            # test
            #     # os.system('perl variant_effect_predictor.pl -i example_GRCh37.vcf --cache --offline --dir_cache vep_cache --fasta vep_cache/homo_sapiens/77_GRCh37/Homo_sapiens.GRCh37.75.dna.primary_assembly.fa')

            # #download plugins
            # os.chdir(vep_cache_dir)
            # if not os.path.isdir('Plugins'):
            #     os.system('git clone %s Plugins' % (vep_plugins_source))

            # #install condel
            # os.chdir('Plugins/config/Condel/config')
            # condel_config = "condel.dir=\\'%s/Plugins/config/Condel\\'" % (vep_cache_dir)
            # os.system('echo %s >> condel_SP.conf' % (condel_config))

    def download_decipher(self):
        os.chdir(data_dir)
        if not os.path.exists('decipher'):
            os.makedirs('decipher')
        os.chdir('decipher')

        if not os.path.isfile(settings.hi_predictions_file):
            command = 'wget -c %s -O %s' % (settings.hi_predictions_source, settings.hi_predictions_file)
            call(command, shell=True)
            # extract
            command = 'gunzip -d %s' % (settings.hi_predictions_file)
            call(command, shell=True)

            file_without_extension = settings.hi_predictions_file.replace('.gz', '')
            # remove first line
            command = 'tail -n +2 "%s" > hi_predictions.unsorted.bed' % (file_without_extension)
            call(command, shell=True)

            command = 'sort -k 1,1 -k2,2n hi_predictions.unsorted.bed > hi_predictions.chr.bed'
            call(command, shell=True)

            # remove chr
            command = "cat hi_predictions.chr.bed | sed 's/^chr//' > hi_predictions.bed"
            call(command, shell=True)

            # compress
            command = 'bgzip hi_predictions.bed'
            call(command, shell=True)
            # rename
            command = 'mv hi_predictions.bed.gz %s' % (settings.hi_predictions_file)
            call(command, shell=True)

            command = 'tabix -p vcf %s' % (settings.hi_predictions_file)
            call(command, shell=True)

            command = 'rm hi_predictions.unsorted.bed %s hi_predictions.chr.bed' % (file_without_extension)
            call(command, shell=True)

        if not os.path.isfile(settings.population_cnv_file):
            command = 'wget -c %s -O %s' % (settings.population_cnv_source, settings.population_cnv_file)
            call(command, shell=True)

        if not os.path.isfile(settings.ddg2p_file):
            command = 'wget -c %s -O %s' % (settings.ddg2p_source, settings.ddg2p_file)
            call(command, shell=True)

    def download_ensembl(self):
        os.chdir(data_dir)
        # download hi_score
        if not os.path.exists('ensembl'):
            os.makedirs('ensembl')
        os.chdir('ensembl')
        if not os.path.isfile(settings.ensembl_phenotype_file):
            command = 'wget -c %s -O %s' % (settings.ensembl_phenotype_source, settings.ensembl_phenotype_file)
            call(command, shell=True)
            command = 'tabix -p vcf %s' % (settings.ensembl_phenotype_file)
            call(command, shell=True)

        if not os.path.isfile(settings.ensembl_clinically_file):
            command = 'wget -c %s -O %s' % (settings.ensembl_clinically_source, settings.ensembl_clinically_file)
            call(command, shell=True)
            command = 'tabix -p vcf %s' % (settings.ensembl_clinically_file)
            call(command, shell=True)

    def download_1000genomes(self):

        os.chdir(data_dir)
        # download 1000genomes
        if not os.path.exists('1000genomes'):
            os.makedirs('1000genomes')
        os.chdir('1000genomes')

        if not os.path.isfile(settings.genomes1k_file):
            command = 'wget -c %s' % (settings.genomes1k_source)
            call(command, shell=True)
            # why do you need to extract ? God only knows ... :P (snpsift merge ?)
            # command = 'gunzip -c %s > %s' % (settings.genomes1k_file, settings.genomes1k_vcf)
            # call(command, shell=True)

        if not os.path.isfile("%s.tbi" % (settings.genomes1k_file)):
            command = 'tabix -p vcf %s' % (settings.genomes1k_file)
            call(command, shell=True)

    def download_dbsnp(self):

        os.chdir(data_dir)

        if not os.path.exists('dbsnp'):
            os.makedirs('dbsnp')
        os.chdir('dbsnp')

        # download
        if not os.path.isfile(settings.dbsnp_file):
            command = 'wget -c %s' % (settings.dbsnp_source)
            call(command, shell=True)

        if not os.path.isfile("%s.tbi" % (settings.dbsnp_file)):
            command = 'wget -c %s.tbi' % (settings.dbsnp_source)
            call(command, shell=True)

        if not os.path.isfile(settings.clinvar_file):
            command = 'wget -c %s' % (settings.clinvar_source)
            call(command, shell=True)

        if not os.path.isfile("%s.tbi" % (settings.clinvar_file)):
            command = 'wget -c %s.tbi' % (settings.clinvar_source)
            call(command, shell=True)

    def download_esp(self):

        os.chdir(data_dir)

        if not os.path.exists('esp6500'):
            os.makedirs('esp6500')

        os.chdir('esp6500')

        if not os.path.isfile(settings.esp_final_file):
            command = 'wget -c %s' % (settings.esp_source)
            call(command, shell=True)

        chroms = list(range(1, 23)) + ['X', 'Y']
        # run prepare_data.sh
        if not os.path.isfile(settings.esp_final_file):
            command = 'tar -zxvf %s' % (settings.esp_file)
            call(command, shell=True)
            vcfs = []
            for chr in chroms:
                command = 'bgzip %s.chr%s.snps_indels.vcf' % (settings.esp_basename, chr)
                call(command, shell=True)
                command = 'tabix -p vcf %s.chr%s.snps_indels.vcf.gz' % (settings.esp_basename, chr)
                call(command, shell=True)
                vcfs.append('%s.chr%s.snps_indels.vcf.gz' % (settings.esp_basename, chr))
            command = 'vcf-concat %s | bgzip -c > esp6500si.vcf.gz' % (" ".join(vcfs))
            call(command, shell=True)
            command = 'tabix -p vcf esp6500si.vcf.gz'
            call(command, shell=True)
            command = 'rm %s* -f' % (settings.esp_basename)
            call(command, shell=True)

    def download_dbnsfp(self):
        os.chdir(data_dir)
        # download dbnsfp

        if not os.path.exists('dbnsfp'):
            os.makedirs('dbnsfp')
        os.chdir('dbnsfp')

        if not os.path.isfile(settings.dbnsfp_file):
            # --user=Anonymous --password=raonyguimaraes@gmail.com
            command = "wget -c %s -O dbNSFPv%s.zip" % (settings.dbnsfp_link, settings.dbnsfp_version)
            call(command, shell=True)

            # Uncompress
            command = 'unzip dbNSFPv%s.zip' % (settings.dbnsfp_version)
            call(command, shell=True)

            # deal with header
            command = """head -n 1 dbNSFP*_variant.chr1 > header.txt """
            call(command, shell=True)

            command = """cat dbNSFP*_variant.chr* | grep -v "^#" > dbNSFP%s.unordered.txt""" % (settings.dbnsfp_version)

            call(command, shell=True)
            # die()

            #use sort -k1,1V -k2,2n for hg38 
            command = """
            mkdir tmp
            sort -k8,8V -k9,9n dbNSFP*.unordered.txt -T tmp/ > dbNSFP%s.ordered.txt
            cat header.txt > dbNSFP%s.txt
            cat dbNSFP*.ordered.txt >> dbNSFP%s.txt
            """ % (settings.dbnsfp_version, settings.dbnsfp_version, settings.dbnsfp_version)
            call(command, shell=True)

            # Compress using block-gzip algorithm

            command = 'bgzip dbNSFP%s.txt' % (settings.dbnsfp_version)
            call(command, shell=True)

            # Create tabix index
            # http://genome.sph.umich.edu/wiki/RareMETALS
            # NOTE: Tabix 1.X does not seem to support the indexing for generic tab-delimited files. To index the file, please use tabix 0.2.5 or earlier versions.
            # use tabix -s 1 -b 2 -e 2 for hg38 and 
            command = 'tabix -s 8 -b 9 -e 9 dbNSFP%s.txt.gz' % (settings.dbnsfp_version)
            call(command, shell=True)

            # clean files
            command = "rm -rf tmp dbNSFP*_variant* dbNSFP*_gene* *ordered.txt *.class *.in *.txt LICENSE.txt try.vcf search_dbNSFP* *.zip"
            # call(command, shell=True)

            # keep original file dbNSFPv3.2a.zip
            # call(command, shell=True)

        if not os.path.isfile(settings.dbscsnv_file):
            command = 'wget %s' % (settings.dbscsnv_source)
            call(command, shell=True)
            command = 'unzip dbscSNV%s.zip' % (settings.dbscsnv_version)
            call(command, shell=True)
            command = 'head -n1 dbscSNV%s.chr1 > h' % (settings.dbscsnv_version)
            call(command, shell=True)
            command = 'cat dbscSNV%s.chr* | grep -v ^chr | cat h - | bgzip -c > dbscSNV%s.txt.gz' % (
                settings.dbscsnv_version, settings.dbscsnv_version)
            call(command, shell=True)
            command = 'tabix -s 1 -b 2 -e 2 -c c dbscSNV%s.txt.gz' % (settings.dbscsnv_version)
            call(command, shell=True)

            command = "rm dbscSNV1.1.zip h *chr*"
            call(command, shell=True)



import os
import subprocess
from subprocess import call, check_output

from .annotator import Annotator
from .install import Installer


class Pynnotator(object):
    """A class to annotate VCFs

    Attributes:
        vcf_file: a VCF file to be annotated
    """

    def __init__(self, vcf_file=False):
        """Return a Pynnotator object with a defined VCF file to be annotated."""
        self.vcf_file = vcf_file

    def install(self):
        print('Starting Installation...')
        installer = Installer()
        installer.install()

        print("Installation Finished with success!! \nNow try testing with the command: pynnotator test")

    def build(self):
        print('Building Databasets...')
        installer = Installer()
        installer.build_datasets()

    def install_libs(self):
        print('Installing Libs...')
        installer = Installer()
        installer.install_libs()

    def install_requirements(self):
        print('Installing Libs...')
        installer = Installer()
        installer.install_requirements()

    def annotate(self, vcf_file):
        print("Annotating VCF... %s" % (vcf_file))
        pynnotator = Annotator(vcf_file)
        pynnotator.run()

    def test(self):
        print('Testing Annotation...')
        path = '%s/tests' % (os.path.dirname(__file__))
        vcf_file = 'sample.1000.vcf.gz'
        os.chdir(path)
        pynnotator = Annotator(vcf_file)
        pynnotator.run()
        os.chdir(path)

        # command = 'grep -v "^#" ann_sample.1000/annotation.final.vcf > result.vcf'
        # call(command, shell=True)

        # compare result with reference
        # try:
        #     command = "zdiff annotation.validated.vcf.gz ann_sample.1000/annotation.final.vcf"
        #     diff = check_output(command, shell=True)
        # except subprocess.CalledProcessError as e:
        #     diff = '1'

        # if diff == b'':
        #     print('Congratulations, The Python Annotation Framework is working as expected, Happy Annotation!!!\n\n')

        command = 'rm -rf ann_sample.1000'
        call(command, shell=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
import subprocess
from datetime import datetime

from pynnotator import settings
from subprocess import run


class SnpSift(object):
    def __init__(self, vcffile=None):
        self.vcffile = vcffile
        self.filename = os.path.splitext(os.path.basename(str(vcffile)))[0]

        # create folder snpeff if it doesn't exists
        if not os.path.exists('snpsift'):
            os.makedirs('snpsift')
            # enter inside folder
            # os.chdir('snpsift')

    def run(self):
        tstart = datetime.now()
        print(tstart, 'Starting snpsift annotation: ', self.vcffile)

        self.annotate()

        tend = datetime.now()
        annotation_time = tend - tstart
        print(tend, 'Finished snpsift annotation, it took: ', annotation_time)

    # convert and annotate the vcf file to snpeff
    def annotate(self):
        # BASE_DIR = os.getcwd()
        # print BASE_DIR

        # dbNFSP
        # command = "%s/java -Xmx40G -jar \
        # %s/SnpSift.jar dbnsfp \
        # -f Uniprot_acc,Uniprot_id,Uniprot_aapos,Interpro_domain,cds_strand,refcodon,SLR_test_statistic,codonpos,fold-degenerate,Ancestral_allele,Ensembl_geneid,Ensembl_transcriptid,aapos,aapos_SIFT,aapos_FATHMM,SIFT_score,SIFT_converted_rankscore,SIFT_pred,Polyphen2_HDIV_score,Polyphen2_HDIV_rankscore,Polyphen2_HDIV_pred,Polyphen2_HVAR_score,Polyphen2_HVAR_rankscore,Polyphen2_HVAR_pred,LRT_score,LRT_converted_rankscore,LRT_pred,MutationTaster_score,MutationTaster_converted_rankscore,MutationTaster_pred,MutationAssessor_score,MutationAssessor_rankscore,MutationAssessor_pred,FATHMM_score,FATHMM_rankscore,FATHMM_pred,RadialSVM_score,RadialSVM_rankscore,RadialSVM_pred,LR_score,LR_rankscore,LR_pred,Reliability_index,VEST3_score,VEST3_rankscore,CADD_raw,CADD_raw_rankscore,CADD_phred,GERP++_NR,GERP++_RS,GERP++_RS_rankscore,phyloP46way_primate,phyloP46way_primate_rankscore,phyloP46way_placental,phyloP46way_placental_rankscore,phyloP100way_vertebrate,phyloP100way_vertebrate_rankscore,phastCons46way_primate,phastCons46way_primate_rankscore,phastCons46way_placental,phastCons46way_placental_rankscore,phastCons100way_vertebrate,phastCons100way_vertebrate_rankscore,SiPhy_29way_pi,SiPhy_29way_logOdds,SiPhy_29way_logOdds_rankscore,LRT_Omega,UniSNP_ids,1000Gp1_AC,1000Gp1_AF,1000Gp1_AFR_AC,1000Gp1_AFR_AF,1000Gp1_EUR_AC,1000Gp1_EUR_AF,1000Gp1_AMR_AC,1000Gp1_AMR_AF,1000Gp1_ASN_AC,1000Gp1_ASN_AF,ESP6500_AA_AF,ESP6500_EA_AF \
        # -db %s \
        # %s \
        # > snpsift/dbnfsp.vcf \
        # "% (java_path, snpsift_dir, dbnsfp, self.vcffile)

        # VARTYPE
        command = "java -Xmx40G -jar \
        %s/SnpSift.jar \
        varType \
       %s \
        > snpsift/snpsift.final.vcf \
        " % (settings.snpeff_dir, self.vcffile)

        # print 'vartype', command

        p = subprocess.call(command,
                            cwd=os.getcwd(),
                            shell=True)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Annotate a VCF File with Snpsift.')

    parser.add_argument('-i', dest='vcffile', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()

    snpsift = SnpSift(args.vcffile)
    snpsift.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
import subprocess
from datetime import datetime
from subprocess import call

from pynnotator import settings

toolname = 'vep'


class Vep(object):
    def __init__(self, vcffile=None):

        self.vcffile = vcffile

        self.filename = os.path.splitext(os.path.basename(str(vcffile)))[0]

        # create folder vep if it doesn't exists
        if not os.path.exists('vep'):
            os.makedirs('vep')
            # enter inside folder
            # os.chdir('vep')

    def install(self):
        print('install')

    def run(self):
        tstart = datetime.now()
        print(tstart, 'Starting vep annotation: ', self.vcffile)

        self.annotate()

        tend = datetime.now()
        annotation_time = tend - tstart
        print(tend, 'Finished vep annotation, it took: ', annotation_time)

    # convert and annotate the vcf file to vep
    def annotate(self):

        command = '''perl %s/vep \
        -i %s \
        --cache \
        --dir %s \
        --offline \
        --format vcf \
        -o vep/vep.output.vcf --vcf --force_overwrite \
        --assembly GRCh37 \
        -sift b -polyphen b \
        -plugin dbNSFP,%s,genename,cds_strand,refcodon,codonpos,codon_degeneracy,Ancestral_allele,AltaiNeandertal,Denisova,Ensembl_geneid,Ensembl_transcriptid,Ensembl_proteinid,aapos,SIFT_score,SIFT_converted_rankscore,SIFT_pred,Uniprot_acc_Polyphen2,Uniprot_id_Polyphen2,Uniprot_aapos_Polyphen2,Polyphen2_HDIV_score,Polyphen2_HDIV_rankscore,Polyphen2_HDIV_pred,Polyphen2_HVAR_score,Polyphen2_HVAR_rankscore,Polyphen2_HVAR_pred,LRT_score,LRT_converted_rankscore,LRT_pred,LRT_Omega,MutationTaster_score,MutationTaster_converted_rankscore,MutationTaster_pred,MutationTaster_model,MutationTaster_AAE,MutationAssessor_UniprotID,MutationAssessor_variant,MutationAssessor_score,MutationAssessor_score_rankscore,MutationAssessor_pred,FATHMM_score,FATHMM_converted_rankscore,FATHMM_pred,PROVEAN_score,PROVEAN_converted_rankscore,PROVEAN_pred,Transcript_id_VEST3,Transcript_var_VEST3,VEST3_score,VEST3_rankscore,MetaSVM_score,MetaSVM_rankscore,MetaSVM_pred,MetaLR_score,MetaLR_rankscore,MetaLR_pred,Reliability_index,M-CAP_score,M-CAP_rankscore,M-CAP_pred,REVEL_score,REVEL_rankscore,MutPred_score,MutPred_rankscore,MutPred_protID,MutPred_AAchange,MutPred_Top5features,CADD_raw,CADD_raw_rankscore,CADD_phred,DANN_score,DANN_rankscore,fathmm-MKL_coding_score,fathmm-MKL_coding_rankscore,fathmm-MKL_coding_pred,fathmm-MKL_coding_group,Eigen_coding_or_noncoding,Eigen-raw,Eigen-phred,Eigen-PC-raw,Eigen-PC-phred,Eigen-PC-raw_rankscore,GenoCanyon_score,GenoCanyon_score_rankscore,integrated_fitCons_score,integrated_fitCons_score_rankscore,integrated_confidence_value,GM12878_fitCons_score,GM12878_fitCons_score_rankscore,GM12878_confidence_value,H1-hESC_fitCons_score,H1-hESC_fitCons_score_rankscore,H1-hESC_confidence_value,HUVEC_fitCons_score,HUVEC_fitCons_score_rankscore,HUVEC_confidence_value,GERP++_NR,GERP++_RS,GERP++_RS_rankscore,phyloP100way_vertebrate,phyloP100way_vertebrate_rankscore,phyloP20way_mammalian,phyloP20way_mammalian_rankscore,phastCons100way_vertebrate,phastCons100way_vertebrate_rankscore,phastCons20way_mammalian,phastCons20way_mammalian_rankscore,SiPhy_29way_pi,SiPhy_29way_logOdds,SiPhy_29way_logOdds_rankscore,1000Gp3_AC,1000Gp3_AF,1000Gp3_AFR_AC,1000Gp3_AFR_AF,1000Gp3_EUR_AC,1000Gp3_EUR_AF,1000Gp3_AMR_AC,1000Gp3_AMR_AF,1000Gp3_EAS_AC,1000Gp3_EAS_AF,1000Gp3_SAS_AC,1000Gp3_SAS_AF,TWINSUK_AC,TWINSUK_AF,ALSPAC_AC,ALSPAC_AF,ESP6500_AA_AC,ESP6500_AA_AF,ESP6500_EA_AC,ESP6500_EA_AF,ExAC_AC,ExAC_AF,ExAC_Adj_AC,ExAC_Adj_AF,ExAC_AFR_AC,ExAC_AFR_AF,ExAC_AMR_AC,ExAC_AMR_AF,ExAC_EAS_AC,ExAC_EAS_AF,ExAC_FIN_AC,ExAC_FIN_AF,ExAC_NFE_AC,ExAC_NFE_AF,ExAC_SAS_AC,ExAC_SAS_AF,ExAC_nonTCGA_AC,ExAC_nonTCGA_AF,ExAC_nonTCGA_Adj_AC,ExAC_nonTCGA_Adj_AF,ExAC_nonTCGA_AFR_AC,ExAC_nonTCGA_AFR_AF,ExAC_nonTCGA_AMR_AC,ExAC_nonTCGA_AMR_AF,ExAC_nonTCGA_EAS_AC,ExAC_nonTCGA_EAS_AF,ExAC_nonTCGA_FIN_AC,ExAC_nonTCGA_FIN_AF,ExAC_nonTCGA_NFE_AC,ExAC_nonTCGA_NFE_AF,ExAC_nonTCGA_SAS_AC,ExAC_nonTCGA_SAS_AF,ExAC_nonpsych_AC,ExAC_nonpsych_AF,ExAC_nonpsych_Adj_AC,ExAC_nonpsych_Adj_AF,ExAC_nonpsych_AFR_AC,ExAC_nonpsych_AFR_AF,ExAC_nonpsych_AMR_AC,ExAC_nonpsych_AMR_AF,ExAC_nonpsych_EAS_AC,ExAC_nonpsych_EAS_AF,ExAC_nonpsych_FIN_AC,ExAC_nonpsych_FIN_AF,ExAC_nonpsych_NFE_AC,ExAC_nonpsych_NFE_AF,ExAC_nonpsych_SAS_AC,ExAC_nonpsych_SAS_AF,gnomAD_exomes_AC,gnomAD_exomes_AN,gnomAD_exomes_AF,gnomAD_exomes_AFR_AC,gnomAD_exomes_AFR_AN,gnomAD_exomes_AFR_AF,gnomAD_exomes_AMR_AC,gnomAD_exomes_AMR_AN,gnomAD_exomes_AMR_AF,gnomAD_exomes_ASJ_AC,gnomAD_exomes_ASJ_AN,gnomAD_exomes_ASJ_AF,gnomAD_exomes_EAS_AC,gnomAD_exomes_EAS_AN,gnomAD_exomes_EAS_AF,gnomAD_exomes_FIN_AC,gnomAD_exomes_FIN_AN,gnomAD_exomes_FIN_AF,gnomAD_exomes_NFE_AC,gnomAD_exomes_NFE_AN,gnomAD_exomes_NFE_AF,gnomAD_exomes_SAS_AC,gnomAD_exomes_SAS_AN,gnomAD_exomes_SAS_AF,gnomAD_exomes_OTH_AC,gnomAD_exomes_OTH_AN,gnomAD_exomes_OTH_AF,gnomAD_genomes_AC,gnomAD_genomes_AN,gnomAD_genomes_AF,gnomAD_genomes_AFR_AC,gnomAD_genomes_AFR_AN,gnomAD_genomes_AFR_AF,gnomAD_genomes_AMR_AC,gnomAD_genomes_AMR_AN,gnomAD_genomes_AMR_AF,gnomAD_genomes_ASJ_AC,gnomAD_genomes_ASJ_AN,gnomAD_genomes_ASJ_AF,gnomAD_genomes_EAS_AC,gnomAD_genomes_EAS_AN,gnomAD_genomes_EAS_AF,gnomAD_genomes_FIN_AC,gnomAD_genomes_FIN_AN,gnomAD_genomes_FIN_AF,gnomAD_genomes_NFE_AC,gnomAD_genomes_NFE_AN,gnomAD_genomes_NFE_AF,gnomAD_genomes_OTH_AC,gnomAD_genomes_OTH_AN,gnomAD_genomes_OTH_AF,clinvar_rs,clinvar_clnsig,clinvar_trait,clinvar_golden_stars,Interpro_domain,GTEx_V6p_gene,GTEx_V6p_tissue \
        --no_progress \
        --no_intergenic \
        --numbers \
        --biotype \
        --total_length \
        --coding_only \
        --pick \
        --symbol \
        1>vep/vep.log \
        --fork %s \
        ''' % (settings.vep_dir, self.vcffile, settings.vep_cache_dir, settings.dbnsfp, settings.vep_cores)
        # 1> vepreport.log \
        ##--pick \
        # condel_plugin, dbscsnv_plugin, 
        # -plugin %s \
        # --plugin %s \

        p = subprocess.call(command,
                            cwd=os.getcwd(),
                            shell=True)

        tend = datetime.now()

        # if p == 0:
        #     print(tend, 'This vcf was sucessfully annotated by %s!' % (toolname))
        # else:
        #     print(tend, 'Sorry this vcf could not be annotated by %s' % (toolname))

        # command = '(grep ^# output.vep.vcf; grep -v ^# output.vep.vcf|sort -k1,1N -k2,2n) > output.vep.sorted.vcf'
        # Sort VCF file 


        command = '''grep '^#' vep/vep.output.vcf > vep/vep.output.sorted.vcf'''
        call(command, shell=True)

        command = '''grep -E -v '^X|^Y|^M|^#|^GL' vep/vep.output.vcf | sort -n -k1 -k2 >> vep/vep.output.sorted.vcf'''
        call(command, shell=True)

        command = '''grep -E '^X' vep/vep.output.vcf | sort -k1,1d -k2,2n >> vep/vep.output.sorted.vcf'''
        call(command, shell=True)

        command = '''grep -E '^Y' vep/vep.output.vcf | sort -k1,1d -k2,2n >> vep/vep.output.sorted.vcf'''
        call(command, shell=True)

        command = '''grep -E '^M' vep/vep.output.vcf | sort -k1,1d -k2,2n >> vep/vep.output.sorted.vcf'''
        call(command, shell=True)

        tend = datetime.now()
        # print(tend, 'Finished sorting vep vcf.')

        command = 'rm vep/vep.output.vcf'
        call(command, shell=True)




if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Annotate a VCF File with VEP.')
    parser.add_argument('-i', dest='vcffile', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()

    vep = Vep(args.vcffile)
    vep.run()



#Gemini wrapper
import argparse
from subprocess import run
from pynnotator import settings
import os

class GnomeAD:
    def __init__(self, vcf, cores):
        self.data = []

    def install(self):
        print('Install GNOMEAD')
        os.chdir('{}/gemini/'.format(settings.libs_dir))
        command = 'wget https://raw.github.com/arq5x/gemini/master/gemini/scripts/gemini_install.py'
        run(command, shell=True)
        if not os.path.exists('gemini'):
            os.makedirs('gemini')
        if not os.path.exists('local'):
            os.makedirs('local')
        command = 'python gemini_install.py local gemini'
        run(command, shell=True)
        command = 'gemini update --dataonly --extra cadd_score'
        run(command, shell=True)
        command = 'gemini update --dataonly --extra gerp_bp'
        run(command, shell=True)
    def main(self):
        #command = '
        print('Annotate GnomeAD')
        command = ''



if __name__ == '__main__':
    
    parser = argparse.ArgumentParser(description='Annotate a VCF File with GEMINI.')

    parser.add_argument('options', help='install test', nargs='?')
    parser.add_argument('-i', dest='vcf_file', required=False, metavar='example.vcf', help='a VCF file to be annotated')
    parser.add_argument('-n', dest='cores', required=False, metavar='4', help='number of cores to use')

    args = parser.parse_args()
    gnomead = GnomeAD(args.vcf_file, args.cores)
    if args.options == 'install':
        gnomead.install()
    else:
        gnomead.main()



import argparse
import logging
import os
import subprocess
from subprocess import check_output

import common

logging.basicConfig(filename='vt.log', level=logging.DEBUG)


class VT():

    def __init__(self, args):
        if args.install:
            self.install()
        self.vt = '{}/vt'.format(common.libs_dir)

    def install(self):
        # change to libs dir

        os.chdir(common.libs_dir)

        if not os.path.exists('vt'):
            command = 'git clone https://github.com/atks/vt'
            output = check_output(command, shell=True)

            logging.info(output)
            os.chdir('vt')
            command = 'make'
            output = check_output(command, shell=True)
            print(output)
            logging.info(output)

    def main(self):
        vt = self.vt
        command = '{}/vt decompose -s {} \
        | {}/vt normalize -r {} - > {}'.format(self.vt, args.input, self.vt, args.reference, args.output)
        output = check_output(command, shell=True, stderr=subprocess.STDOUT).decode('utf-8')
        print(output)
        logging.info(output)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Decompose and normalize a VCF file.')
    parser.add_argument('--install', dest='install', action='store_true')
    parser.add_argument('--reinstall', dest='reinstall', action='store_true')
    parser.add_argument('-i', dest='input', required=False, metavar='sample.vcf', help='a VCF file to be annotated')
    parser.add_argument('-r', dest='reference', required=False, metavar='reference.fasta', help='a reference fasta')
    parser.add_argument('-o', dest='output', required=False, metavar='reference.fasta', help='a reference fasta')
    args = parser.parse_args()

    vt = VT(args)
    vt.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
import subprocess
from datetime import datetime
from subprocess import call, run

from pynnotator import settings

toolname = 'decipher'

# enable perl5_lib
env = os.environ.copy()
env['PERL5LIB'] = settings.vcftools_dir_perl


class Decipher(object):
    def __init__(self, vcffile=None):

        self.vcffile = vcffile
        self.filename = os.path.splitext(os.path.basename(str(vcffile)))[0]

        # create folder decipher if it doesn't exists
        if not os.path.exists('decipher'):
            os.makedirs('decipher')

    def run(self):

        tstart = datetime.now()
        print(tstart, 'Starting decipher annotation: ', self.vcffile)

        self.annotate()

        tend = datetime.now()
        annotation_time = tend - tstart
        print(tend, 'Finished decipher annotation, it took: ', annotation_time)

    # Annotate vcf file with Haploinsuficiency Index
    def annotate(self):

        command = '%s/bgzip -c %s > decipher/%s.vcf.gz' % (settings.htslib_dir, self.vcffile, self.filename)
        call(command, shell=True)

        command = '%s/tabix -p vcf decipher/%s.vcf.gz' % (settings.htslib_dir, self.filename)
        call(command, shell=True)

        command = '''zcat decipher/%s.vcf.gz | %s/vcf-annotate \
        -a %s \
        --fill-type \
        -d key=INFO,ID=HI_PREDICTIONS,Number=.,Type=String,Description=\'Haploinsuficiency Predictions\' \
        -c CHROM,FROM,TO,INFO/HI_PREDICTIONS > decipher/hi_predictions.vcf \
        ''' % (self.filename, settings.vcftools_dir_perl, settings.hi_predictions)

        run(command, shell=True)
        # print command

        # p = subprocess.call(command,
        #                     cwd=os.getcwd(),
        #                     env=env,
        #                     shell=True)

        time_end = datetime.now()

        command = 'rm %s/checked.vcf.gz*' % (toolname)
        # run(command, shell=True)


        # if p == 0:
        #     print(time_end, 'This vcf was sucessfully annotated by %s!' % (toolname))
        # else:
        #     print(time_end, 'Sorry this vcf could not be annotated by %s' % (toolname))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Annotate a VCF File with Decipher.')

    parser.add_argument('-i', dest='vcffile', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()

    decipher = Decipher(args.vcffile)
    decipher.run()



from subprocess import run
#options
#use vcfanno
#use gatk
#benchmark ?
command = 'java -Xmx15g -jar dist/GenomeAnalysisTK.jar \
-T CombineVariants \
-R /home/raony/dev/pynnotator/pynnotator/data/vep_cache/homo_sapiens/90_GRCh37/Homo_sapiens.GRCh37.75.dna.primary_assembly.fa.gz \
'

command = '/home/raony/dev/pynnotator/pynnotator/libs/gatk/gatk/gatk-launch MergeVcfs \
-I /home/raony/dev/pynnotator/pynnotator/tests/merge/snpeff.output.vcf \
-I /home/raony/dev/pynnotator/pynnotator/tests/merge/vep.output.sorted.vcf \
-O union.vcf'

run(command, shell=True)

#https://gatkforums.broadinstitute.org/gatk/discussion/7515/simple-question-about-combinevariants
# $GATK -T CombineVariants \
# -R $REFERENCE_HG38 \
# --variant:strelka vcf-strelka.vcf \
# --variant:mutect2 vcf-mutect2.vcf \
# -o vcf-combine.vcf \
# -genotypeMergeOptions PRIORITIZE \
# -priority mutect2,strelka \
# --disable_auto_index_creation_and_locking_when_reading_rods

#https://gatkforums.broadinstitute.org/gatk/discussion/53/combining-variants-from-different-files-into-one
#java -Xmx2g -jar dist/GenomeAnalysisTK.jar -T CombineVariants -R bundle/b37/human_g1k_v37.fasta -L 1:1-1,000,000 -V:omni bundle/b37/1000G_omni2.5.b37.sites.vcf -V:hm3 bundle/b37/hapmap_3.3.b37.sites.vcf -o union.vcf



import os
from subprocess import run
from pynnotator import settings
import shutil

shutil.rmtree(settings.vcf_validator_dir)
os.makedirs(settings.vcf_validator_dir)
os.chdir(settings.vcf_validator_dir)

command = 'wget https://github.com/EBIvariation/vcf-validator/releases/download/v0.7/vcf_validator'
run(command,shell=True)
command = 'chmod +x vcf_validator'
run(command,shell=True)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
import subprocess
from datetime import datetime

from pynnotator import settings
from subprocess import run

class Snpeff(object):
    def __init__(self, vcffile=None):

        self.vcffile = vcffile
        self.filename = os.path.splitext(os.path.basename(str(vcffile)))[0]

        # create folder snpeff if it doesn't exists
        if not os.path.exists('snpeff'):
            os.makedirs('snpeff')

    def run(self):
        tstart = datetime.now()
        print(tstart, 'Starting snpEff annotation: ', self.vcffile)

        self.annotate()

        tend = datetime.now()
        annotation_time = tend - tstart
        print(tend, 'Finished snpEff annotation, it took: ', annotation_time)
        

    # convert and annotate the vcf file to snpeff
    def annotate(self):

        BASE_DIR = os.getcwd()
        # print BASE_DIR
        # -canon to report only canonical transcript, -o gatk to report only one #GRCh37.64
        # true
        # snpeff 4.0
        command = """java -Xmx%s -jar %s/snpEff.jar \
        -c %s/snpEff.config \
        %s %s \
        -no-downstream \
        -no-intergenic \
        -no intragenic \
        -onlyProtein \
        -no-intron \
        -no-upstream \
        -noNextProt \
        -no-utr -canon \
        >snpeff/snpeff.output.vcf""" % (
            settings.snpEff_memory, settings.snpeff_dir, settings.snpeff_dir, settings.snpeff_database, self.vcffile)
        # print(command)


        # 

        p = subprocess.call(command,
                            cwd=os.getcwd(),
                            shell=True)

        tend = datetime.now()

        command = 'mv snpEff_genes.txt snpEff_summary.html snpeff/'
        run(command, shell=True)
        
        # if p == 0:
        #     print(tend, 'This vcf was annotated by snpEff with Success.')
        # else:
        #     print(tend, 'Sorry this vcf could not be anotated by snpeff.')
        return p


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Annotate a VCF File with Snpeff.')

    parser.add_argument('-i', dest='vcffile', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()

    snpeff = Snpeff(args.vcffile)
    snpeff.run()



from subprocess import run

# test snpeff annotation
path = '../libs/htslib/htslib-1.5'
command = '%s/bgzip sample.1000.vcf -c > sample.1000.vcf.gz' % (path)
run(command, shell=True)
command = '%s/tabix -p vcf sample.1000.vcf.gz' % (path)
run(command, shell=True)



from setuptools import setup

setup(
      name='pynnotator',
      version='1.6',
      description='A Python Annotation Framework for VCFs using multiple tools (Ex. VEP, SnpEff and SnpSift) and databases (Ex. 1000genomes, dbSNP and dbnfsp) .',
      url='http://github.com/raonyguimaraes/pynnotator',
      author='Raony Guimaraes',
      author_email='raony@torchmed.com',
      classifiers=[
            # How mature is this project? Common values are
            #   3 - Alpha
            #   4 - Beta
            #   5 - Production/Stable
            'Development Status :: 4 - Beta',

            # Indicate who your project is intended for
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Build Tools',

            # Pick your license as you wish (should match "license" above)
            'License :: OSI Approved :: BSD License',

            # Specify the Python versions you support here. In particular, ensure
            # that you indicate whether you support Python 2, Python 3 or both.
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.6',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.2',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Topic :: Scientific/Engineering :: Bio-Informatics'
      ],
      keywords='genome exome annotation rare diseases',
      license='BSD-3',
      packages=['pynnotator', 'pynnotator.helpers', 'pynnotator.tests'],
      install_requires=[
          'wheel',
          'pysam',
          'cython'
      ],
      test_suite='nose.collector',
      tests_require=['nose'],
      #scripts=['bin/pynnotator'],
      entry_points={  # Optional
              'console_scripts': [
                  'pynnotator=pynnotator.main:main',
              ],
      },
      include_package_data=True,
      zip_safe=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
import pysam
from collections import OrderedDict
from datetime import datetime

from pynnotator import settings

toolname = 'merge'


class Merge(object):
    def __init__(self, vcffile=None):

        self.vcffile = vcffile

        self.filename = os.path.splitext(os.path.basename(str(vcffile)))[0]

        self.header = open('%s/scripts/header.vcf' % (settings.BASE_DIR)).readlines()

        # create folder merge if it doesn't exists
        if not os.path.exists('merge'):
            os.makedirs('merge')
        # enter inside folder
        os.chdir('merge')

        self.annotation_files = OrderedDict()

        pysam.tabix_index('../snpeff/snpeff.output.vcf', preset='vcf')

        self.annotation_files['snpeff'] = {
            'info': 'EFF',
            'file': pysam.Tabixfile('../snpeff/snpeff.output.vcf.gz', 'r', encoding="utf-8")
        }

        pysam.tabix_index('../vep/vep.output.sorted.vcf', preset='vcf')

        self.annotation_files['vep'] = {
            'info': 'CSQ',
            'file': pysam.Tabixfile('../vep/vep.output.sorted.vcf.gz', 'r', encoding="utf-8")
        }

        pysam.tabix_index('../snpsift/snpsift.final.vcf', preset='vcf')

        self.annotation_files['vartype'] = {
            'info': 'VARTYPE,SNP,MNP,INS,DEL,MIXED,HOM,HET',
            'file': pysam.Tabixfile('../snpsift/snpsift.final.vcf.gz', 'r', encoding="utf-8")
        }

        self.dbsnp = pysam.Tabixfile(settings.dbsnp, 'r', encoding="utf-8")

    def run(self):
        tstart = datetime.now()
        print(tstart, 'Starting merge: ', self.vcffile)

        self.merge()

        tend = datetime.now()
        annotation_time = tend - tstart
        print(tend, 'Finished merge, it took: ', annotation_time)

        # merge VCF Files with VariantAnnotator

    def merge(self):

        BASE_DIR = os.getcwd()

        # get headers from files
        headers = []
        for annotation in self.annotation_files:
            # print(annotation, self.annotation_files[annotation])
            info = self.annotation_files[annotation]['info'].split(',')
            # print(info)
            ann_vcf_file = self.annotation_files[annotation]['file']

        output = open('annotation.final.vcf', 'w')
        input_vcf = open(self.vcffile)
        for line in input_vcf:
            if line.startswith('#'):
                if line.startswith('#CHROM'):
                    # write extra header
                    # for tag in headers:
                    #     # print(tag)
                    #     output.writelines(tag + '\n')
                    output.writelines(self.header)
                    output.writelines(line)
                else:
                    output.writelines(line)
            else:
                # here annotation comes!
                variant = line.split('\t')
                variant[0] = variant[0].replace('chr', '')
                index = '%s-%s' % (variant[0], variant[1])
                # print(variant)
                annotations = []
                for annotation in self.annotation_files:
                    info = self.annotation_files[annotation]['info'].split(',')
                    # print(info)
                    ann_vcf_file = self.annotation_files[annotation]['file']
                    try:
                        records = ann_vcf_file.fetch(variant[0], int(variant[1]) - 1, int(variant[1]))
                    except:
                        records = []
                    for record in records:

                        ann = record.split('\t')
                        # print(ann)
                        new_info = ann[7].split(';')

                        if annotation == 'pynnotator':
                            for tag in new_info:
                                string = tag.split('=')[0].split('.')[0]
                                if string in self.pynnotator_tags:
                                    annotations.append(tag)
                        else:
                            # print(info)
                            for tag in new_info:
                                string = tag.split('=')[0]
                                if string in info:
                                    annotations.append(tag)
                                    # print(annotations)

                new_info = variant[7] + ';' + ";".join(annotations)
                # print(new_info)
                variant[7] = new_info

                # now dbsnp

                try:
                    records = self.dbsnp.fetch(variant[0], int(variant[1]) - 1, int(variant[1]))
                except:
                    records = []
                rsids = []
                for record in records:
                    # print(record, variant)
                    row = record.split('\t')
                    rsid = row[2]

                    check_flag = False
                    if variant[3] == row[3]:
                        alts = variant[4].split(',')
                        alts_row = row[4].split(',')
                        # compare ALT
                        for alt in alts:
                            if alt in alts_row:
                                check_flag = True
                    if check_flag:
                        rsids.append(rsid)

                if len(rsids) > 0:
                    variant[2] = ",".join(rsids)

                new_line = "\t".join(variant)
                output.writelines(new_line)
                # index()
        output.close()


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='MERGE all VCF Files from previous methods.')
    parser.add_argument('-i', dest='vcffile', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()

    merge = Merge(args.vcffile)
    merge.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# parallel https://code.google.com/p/pysam/issues/detail?id=105


# import pp

import argparse
import multiprocessing as mp
import os
import pysam
from datetime import datetime

# import shlex, subprocess
from pynnotator import settings


# cores = int(args.cores)
# prefix = 'func_pred'
# if not os.path.exists(prefix):
#     os.makedirs(prefix)

class FUNC_PRED_Annotator(object):
    def __init__(self, vcf_file=None, cores=None):

        self.vcf_file = vcf_file
        self.dbnfsp_header = open('%s/dbnsfp/header.vcf' % (settings.data_dir), 'r', encoding="utf-8").readlines()

        # print('self.resources', self.resources)
        self.cores = int(cores)

        self.filename = os.path.splitext(os.path.basename(str(vcf_file)))[0]
        # create folder validator if it doesn't exists
        if not os.path.exists('func_pred'):
            os.makedirs('func_pred')

    def run(self):

        tstart = datetime.now()

        print(tstart, 'Starting func pred annotator: ', self.vcf_file)

        # std = self.annotator()

        self.splitvcf(self.vcf_file)

        pool = mp.Pool()
        pool.map(self.annotate, range(1, self.cores + 1))
        # pool.close()
        # pool.join()

        prefix = 'func_pred'
        # # Define your jobs
        # jobs = []
        final_parts = []
        for n in range(0, self.cores):
            index = n + 1
            final_file = 'func_pred/func_pred.%s.vcf' % (index)
            final_parts.append(final_file)

        command = 'cat %s/header.vcf ' % (prefix) + " ".join(final_parts) + '> %s/func_pred.vcf' % (prefix)
        std = os.system(command)

        command = 'cat %s/func_pred.vcf | vcf-sort > %s/func_pred_sorted.vcf' % (prefix, prefix)
        std = os.system(command)

        tend = datetime.now()
        annotation_time = tend - tstart
        print(tend, 'Finished func pred, it took: ', annotation_time)

    def partition(self, lst, n):
        division = len(lst) / float(n)
        return [lst[int(round(division * i)): int(round(division * (i + 1)))] for i in range(n)]

    def splitvcf(self, vcffile):
        # print('split file', vcffile)
        # print 'numero de cores', cores
        prefix = 'func_pred'
        vcf_reader = open('%s' % (vcffile), encoding='utf-8')
        header_writer = open('%s/header.vcf' % (prefix), 'w')
        body_writer = open('%s/body.vcf' % (prefix), 'w')

        count_lines = 0
        for line in vcf_reader:
            if line.startswith('#'):
                if line.startswith('#CHROM'):
                    header_writer.writelines(self.dbnfsp_header)
                header_writer.writelines(line)
            else:
                body_writer.writelines(line)
        
        header_writer.close()
        body_writer.close()

        vcf_reader = open('%s/body.vcf' % (prefix), encoding="utf-8")

        groups = self.partition(list(vcf_reader.readlines()), self.cores)
        for c, group in enumerate(groups):
            # print 'group', len(group)
            # print 'c', c
            part = c + 1
            part_writer = open('%s/part.%s.vcf' % (prefix, part), 'w')
            for line in group:
                part_writer.writelines(line)
            part_writer.flush()
            os.fsync(part_writer.fileno())
            part_writer.close()
            

    # convert and annotate the vcf file to snpeff
    def annotate(self, out_prefix):
        # print 'Hello'
        # print self.dbnfsp_reader
        # header is at:

        # 24    SIFT_score: SIFT score (SIFTori).
        # 105 HUVEC_confidence_value: 0 - highly significant scores (approx. p<.003); 1 - significant scores

        # 188   clinvar_rs: rs number from the clinvar data set
        # 191 clinvar_golden_stars: ClinVar Review Status summary.

        func_pred_start = 23
        func_pred_end = 105

        clinvar_start = 187
        clinvar_end = 191

        # print 'input',vcffile, out_prefix, dbnsfp 
        dbnfsp_reader = pysam.Tabixfile(settings.dbnsfp, 'r', encoding='utf-8')

        # print('header')
        for item in str(dbnfsp_reader.header):
            header = item.strip().split('\t')

        # header = dbnfsp_reader.header.next().strip().split('\t')

        vcffile = 'func_pred/part.%s.vcf' % (out_prefix)

        vcf_reader = open('%s' % (vcffile), encoding='utf-8')
        vcf_writer = open('func_pred/func_pred.%s.vcf' % (out_prefix), 'w')

        for line in vcf_reader:
            if line.startswith('#'):
                if line.startswith('#CHROM'):
                    vcf_writer.writelines(dbnfsp_header)
                vcf_writer.writelines(line)
            else:
                variant = line.split('\t')
                variant[0] = variant[0].replace('chr', '')
                index = '%s-%s' % (variant[0], variant[1])
                # print index
                try:
                    records = dbnfsp_reader.fetch(variant[0], int(variant[1]) - 1, int(variant[1]))
                except:
                    records = []

                for record in records:
                    ann = record.strip().split('\t')

                    ispresent = False
                    if variant[3] == ann[2]:
                        alts = variant[4].split(',')
                        alts_ann = ann[3].split(',')
                        # compare ALT
                        for alt in alts:
                            if alt in alts_ann:
                                ispresent = True

                    if ispresent:
                        new_ann = []

                        for k, item in enumerate(header[func_pred_start:func_pred_end]):
                            idx = k + func_pred_start
                            if ann[idx] != '.':
                                new_ann.append('dbNSFP_%s=%s' % (item, ann[idx].replace(';', '|')))

                        for k, item in enumerate(header[clinvar_start:clinvar_end]):
                            idx = k + clinvar_start
                            if ann[idx] != '.':
                                new_ann.append('dbNSFP_%s=%s' % (item, ann[idx].replace(';', '|').replace(' ', '_')))

                        variant[7] = '%s;%s' % (variant[7], ";".join(new_ann))
                vcf_writer.writelines("\t".join(variant))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Annotate a VCF File with DbNSFP.')

    parser.add_argument('-i', dest='vcf_file', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    parser.add_argument('-n', dest='cores', required=True, metavar='4', help='number of cores to use')

    args = parser.parse_args()

    func_pred = FUNC_PRED_Annotator(args.vcf_file, args.cores)
    func_pred.run()

